
#ifndef _AX25_H_
#define _AX25_H_

#include "queue.h"
#include "ax25_l1.h"
#include "ax25_l2.h"
#include "ax25_io.h"

/*
 *	Controlblock (struct AX25_cb) Typen. Verblueffende
 *	Aehnlichkeiten mit den SOCKET-Typen ;-)
 *
 */
#define TYPE_STREAM     0
#define TYPE_DGRAM	1
#define TYPE_RAW	2
#define TYPE_SEQPACKET	3

/*
 *	Unterschiedliche Zustaende eines AX.25-Controlblocks
 *	AX25_cb::state
 *
 */
#define AX25_IDLE		0
#define AX25_CONNECTING		1
#define AX25_LISTENING		2
#define AX25_ESTABLISHED	3
#define AX25_LINGER		4

/*
 *	Events, die vom AX.25-Stack an das hoehere Layer
 *	gemeldet werden koennen.
 *
 */
#define EVENT_NOEVENT		0
#define EVENT_CONNECT		1	/* connect */
#define EVENT_CONTOUT		2	/* connect timed out */
#define EVENT_CONREFUSED	3	/* connection refused */
#define EVENT_CONREQ		4	/* connection request */
#define EVENT_LINKFAILURE	5	/* link failure */
#define EVENT_LINKRESET		6	/* link reset */
#define EVENT_DISCONNECT	7	/* disconnect */
#define EVENT_TXQEMPTY		8	/* Tx-Queue empty */

/*
 *	Der AX.25-Controlblock. In dieser Struktur befinden sich
 *	alle Daten, die fuer den Betrieb einer AX.25-Verbindung
 *	benoetigt werden.
 *
 */
struct AX25_cb {
	struct AX25_cb xdata	*prev;
	struct AX25_cb xdata	*next;

	unsigned char		state;	
	unsigned char		type;

	struct AX25_path	path;
	struct AX25_l2data	l2;

	struct AX25_io		io;
	
	struct Queue		inq;
	struct Queue		outq;

	char			event;
};


/*
 *	Funktionen fuer das high-layer
 *
 */
struct AX25_cb xdata * ax25_open (unsigned char type);
void ax25_close (struct AX25_cb xdata * axcb);
char ax25_connect (struct AX25_cb xdata * axcb, 
                   struct AX25_smart_path xdata * path);
char ax25_listen (struct AX25_cb xdata * axcb);
char ax25_accept (struct AX25_cb xdata * axcb);
char ax25_event (struct AX25_cb xdata * axcb);

#endif

