
/*
 *	$Id: ax25_l1.c,v 1.1.1.1 2002/02/15 22:36:40 Klaus Exp $
 *	
 *	ax25_l1.c	AX.25-Layer 1
 *
 *			Funktionen zum Decodieren und Codieren von
 *			AX.25-Bloecken. 
 *
 *
 *
 *	(c) 1999,	Klaus Hennemann, DL3KHB	
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "mm.h"
#include "buf.h"
#include "ax25.h"
#include "ax25_sup.h"
#include "ax25_l1.h"
#include "ax25_l2.h"
#include "ax25_dg.h"
#include "kiss.h"
#include "string.h"


/*
 *	NOCALL in Form einer Adress-Instanz
 *
 */
struct AX25_address code l1_nocall = {
	{ 'R' << 1, 'P' << 1, 'C' << 1, ' ' << 1, ' ' << 1, ' ' << 1 },
	
	AX25_MASK_RR
};


/*
 *	Eigene AX.25-Adresse
 *
 */
struct AX25_address xdata l1_mycall; 


/*---------------------------------------------------------------------------*/
/*									     */
/*		Layer 1 Sendefunktionen					     */
/*									     */
/*---------------------------------------------------------------------------*/


/*
 *	Extrahiert den AX.25-Pfad aus dem Puffer <buf>, berechnet den
 *	Rueckpfad und speichert diesen in <axpath>.
 *
 */
static void l1_mk_rpath (struct AX25_path xdata *axpath,
		         unsigned char xdata *buf)
{
	unsigned char data n;
	unsigned char xdata * idata p;

	p = axpath->path.path;

	for (n = 0; n < AX25_CALL_SIZE; n++)
		*p++ = *buf++;

	*p++ = (*buf & ~AX25_MASK_END) | AX25_MASK_RR;

	for (n = 0; n < AX25_CALL_SIZE; n++)
		*p++ = l1_mycall.call[n];

	if (*buf & AX25_MASK_END) {
		*p = l1_mycall.ssid | AX25_MASK_END | AX25_MASK_RR;
		axpath->path_size = 2 * (AX25_CALL_SIZE + 1);
		return;
	}

	*p = (l1_mycall.ssid & ~AX25_MASK_END) | AX25_MASK_RR;

	for (n = 1; n <= AX25_DIGIS_MAX; n++) {
		buf += (AX25_CALL_SIZE + 1);

		if (*buf & AX25_MASK_END)
			break;
	}
	
	axpath->path_size = (2 + n) * (AX25_CALL_SIZE + 1);
		
	buf -= AX25_CALL_SIZE;

	for (; n != 0; n--) {
		unsigned char data j;

		p++;
		
		for (j = 0; j < AX25_CALL_SIZE; j++)
			*p++ = *buf++;

		*p = (*buf & ~(AX25_MASK_END | AX25_MASK_H)) | AX25_MASK_RR;

		buf -= 2 * AX25_CALL_SIZE + 1;
	}

	*p |= AX25_MASK_END;
}


/*
 *	Baut einen AX.25-Block zusammen und sendet ihn aus.
 *
 *	Im Fehlerfall (nicht gesendet) wird 0 zurueckgegeben, ansonsten 1
 *
 */
char l1_xmit (struct AX25_path xdata *axpath,
	      unsigned char ctl,
	      unsigned char ns,
	      unsigned char nr,
	      unsigned char flag,
	      unsigned char pid,
	      struct Buffer xdata *buf)
{
	struct Buffer xdata * idata blk;
	unsigned char data b;

	if ( (blk = buf_new(AX25_BLK_MAX + 1)) == NULL )
		return 0;

	b = axpath->path.addr.peer.ssid;
	b = (b & ~AX25_MASK_C) | (flag & AX25_COMMAND);
	axpath->path.addr.peer.ssid = b;

	b = axpath->path.addr.local.ssid;
	b = (b & ~AX25_MASK_C) | (~flag & AX25_COMMAND);
	axpath->path.addr.local.ssid = b;

	buf_append(blk,axpath->path.path,axpath->path_size);
	buf_putch(blk,ctl | (nr << 5) | (ns << 1) | (flag & AX25_FINAL));
	
	if (ctl == AX25_I || ctl == AX25_UI) {
		buf_putch(blk,pid);

		if (buf != NULL)
			buf_append(blk,buf_data(buf),buf_size(buf));
	}

	if (buf_size(blk) > AX25_BLK_MAX || !kiss_xmit(blk)) {
		buf_delete(blk);
		return 0;
	}

	return 1;
}


/*
 *	Schickt ein DM an den Absender zurueck.
 *
 */
static void l1_return_dm (unsigned char xdata *buf,unsigned char flag)
{
	struct AX25_path xdata axp;

	l1_mk_rpath(&axp,buf);
	l1_xmit_dm(&axp,INVERT_FLAG(flag));
}


/*---------------------------------------------------------------------------*/
/*									     */
/*		Layer 1 Empfangsfunktionen				     */
/*									     */
/*---------------------------------------------------------------------------*/


/*
 *	Vergleicht die beiden AX.25-Adressen *a1 und *a2. Wenn die 
 *	Calls total unterschiedlich sind, wird -1 zurueckgegeben. Wenn
 *	nur die SSIDs unterschiedlich sind, wird 0 zurueckgegeben. Bei
 *	Gleichheit wird 1 zurueckgegeben.
 *
 */
char l1_cmpaddr (struct AX25_address xdata *a1,struct AX25_address xdata *a2)
{
	register char idata n;

	/*
	 *	Es ist am wahrscheinlichsten, dass sich zwei Rufzeichen
	 *	in den letzten Stellen unterscheiden. Daher wird hier
	 *	rueckwaerts verglichen!
	 *
	 */
	for (n = sizeof(a1->call) - 1; n >= 0; n--)
		if (a1->call[n] != a2->call[n])
			return -1;

	if ( (a1->ssid & AX25_MASK_SSID) != (a2->ssid & AX25_MASK_SSID) )
		return 0;
	
	return 1;
}


/*
 *	Ueberliesst den kompletten Digipeaterpfad und prueft, ob
 *	bei allen Digipeatern das H-Bit gesetzt ist.
 *	Wenn der Digi-Pfad korrekt ist wird 1 zurueckgegeben, ansonsten
 *	0.
 *
 */
static char l1_skip_path (struct Buffer xdata *buf)
{
	struct AX25_address xdata * idata axaddr;
	unsigned char idata n;

	for (n = 0; n < AX25_DIGIS_MAX; n++) {
		axaddr = buf_pull(buf,sizeof(struct AX25_address));

		if (axaddr == NULL)
			break;

		if ( !(axaddr->ssid & AX25_MASK_H) )
			break;

		if (axaddr->ssid & AX25_MASK_END)
			return 1;
	}

	return 0;
}


/*
 *	Nimmt alle empfangenen AX.25-Frames entgegen und wertet
 *	sie aus.
 *
 */
void l1_recv (struct Buffer xdata *buf)
{
	struct AX25_cb xdata * idata axcb;
	struct AX25_address xdata * idata dest;
	struct AX25_address xdata * idata src;
	struct AX25_block idata axb;
	int idata b;

	if (buf_size(buf) < 2 * sizeof(struct AX25_address) + 1)
		goto skip;

	dest = buf_pull(buf,sizeof(struct AX25_address));
	
	if (l1_cmpaddr(&l1_mycall,dest) != 1)
		goto skip;

	src = buf_pull(buf,sizeof(struct AX25_address));

	if ( !(src->ssid & AX25_MASK_END) )
		if (!l1_skip_path(buf))
			goto skip;
	
	/* AX.25 Version 1? */
	if (!((dest->ssid ^ src->ssid) & AX25_MASK_C))
		goto skip;

	if ( (b = buf_getch(buf)) < 0 )
		goto skip;

	axb.ctl_raw = (unsigned char)b;

	/* Kommando decodieren */
	if ( !(axb.ctl_raw & 0x01) )		/* I-Block? */
		axb.ctl = AX25_I;
	else if ( !(axb.ctl_raw & 0x02) )	/* S-Block? */
		axb.ctl = axb.ctl_raw & 0x0f;
	else					/* U-Block! */
		axb.ctl = axb.ctl_raw & 0xef;

	/* Kommando-Flag (Poll, Final, Command, Response) decodieren */
	axb.flag = (dest->ssid & AX25_COMMAND) | (axb.ctl_raw & AX25_FINAL); 

	/* NR und NS decodieren */
	axb.ns = (axb.ctl_raw >> 1) & 0x07;
	axb.nr = axb.ctl_raw >> 5;

	/* PID-ermitteln, falls I- oder UI-Block */
	if (axb.ctl == AX25_I || axb.ctl == AX25_UI) {
		if ( (b = buf_getch(buf)) < 0 )
			goto skip;

		axb.pid = (unsigned char)b;
	}

	axb.buf = buf;

	/*
	 *	Nur ein Datagramm?
	 *
	 */
	if (axb.ctl == AX25_UI) {
		if ( !(axcb = ax25_lookup(src,TYPE_DGRAM)) ||
		     axcb->state != AX25_ESTABLISHED )
			goto skip;
		
		dg_recv(axcb,&axb);
		return;
	}
	
	/* 
	 * 	Wenn ein passender Control-Block existiert, AX.25-Block
	 * 	an das Layer 2 weitergeben
	 */
	if ( (axcb = ax25_lookup(src,TYPE_SEQPACKET)) ) {
		l2_recv(axcb,&axb);
		return;
	}

	/*
	 *	Ohh, ein Connect-Wunsch. Wenn wir ein Listen-CB
	 *	haben wird der Connect-Wunsch weitergeleitet, 
	 *	ansonsten abgelehnt.
	 *
	 */
	if (axb.ctl == AX25_SABM) {
		if ( !(axcb = ax25_listener()) )
			goto refuse;

		l1_mk_rpath(&axcb->path,(unsigned char xdata *)src);
		l2_recv(axcb,&axb);
		return;
	}

	if (axb.ctl == AX25_DM)
		goto skip;
		
refuse:
	l1_return_dm((unsigned char xdata *)src,axb.flag);
skip:
	buf_delete(buf);	
}


/*---------------------------------------------------------------------------*/
/*									     */
/*		Layer 1 Zusatzfunktionen				     */
/*									     */
/*---------------------------------------------------------------------------*/


/* 
 *	Bereitet den Pfad <p> zur Verwendung im Layer 1 vor.
 *	Dabei werden bei allen Digipeatern die H-Bits geloescht, die
 *	RR-Bits gesetzt	und die Adress-Ende-Kennung korrekt gesetzt.
 *
 */
void l1_prepare_path (struct AX25_path xdata *p)
{
	unsigned char xdata *ptr;

	ptr = p->path.path + AX25_CALL_SIZE;

	while (ptr < p->path.path + p->path_size) {
		*ptr = (*ptr & ~(AX25_MASK_END | AX25_MASK_H)) | AX25_MASK_RR;
		ptr += AX25_CALL_SIZE + 1;
	}

	ptr -= AX25_CALL_SIZE + 1;
	*ptr |= AX25_MASK_END;
}


/*
 *	Wandelt einen Smart-Path in einen Path
 *
 */
void l1_mkpath (struct AX25_path xdata *p, struct AX25_smart_path xdata *sp)
{
	memcpy_xx(&p->path.addr.peer, 
	          &sp->peer, 
		  sizeof(sp->peer));
		  
	memcpy_xx(&p->path.addr.local, 
	          &l1_mycall, 
		  sizeof(l1_mycall));
		  
	memcpy_xx(p->path.addr.digi, 
	          sp->digi, 
		  sizeof(sp->digi));

	p->path_size = sizeof(struct AX25_address) * (2 + sp->digi_cnt);
}


/*
 *	Setzt das eigene Rufzeichen
 *
 */
void l1_set_mycall (struct AX25_address xdata *mycall)
{
	memcpy_xx(&l1_mycall,mycall,sizeof(struct AX25_address));
	l1_mycall.ssid |= AX25_MASK_RR;
}


