
#ifndef _BOOT_H_
#define _BOOT_H_

#include "flash.h"
#include "ax25.h"

/*
 *	Adresse im EEPROM, ab der die Boot-Konfiguration beginnt.
 *
 */
#define BOOT_CONF_ADDR	0x0000

/*
 *	Magic-Number der Boot-Konfiguration
 *
 */
#define BOOT_CONF_MAGIC	0xbcbc

/*
 *	Die Magic-Number der Firmware
 *
 */
#define FW_MAGIC	0x1bc2


/*
 *	Baenke, die gebootet werden koennen
 *
 */
#define BOOT_FLASH0	FLASH_BANK0
#define BOOT_FLASH1	FLASH_BANK1
#define BOOT_EPROM	64	
 

/*
 *	Daten einer Flash-Bank
 *
 */
struct Bank {
	unsigned char	state;		/* Bank-Zustand */
	unsigned int	crc;		/* THP-CRC */
	unsigned int	start_cnt;	/* Anzahl Firmware-Starts */

	unsigned char	version_major;	/* Firmware-Version */
	unsigned char	version_minor;
	unsigned char	version_patch;
};


/*
 *	Boot-Konfiguration, die im EEPROM gespeichert ist.
 *
 */
struct BootConf {
	/*
	 *	Magic-Number der Boot-Konfiguration
	 *
	 */
	unsigned int	magic;

	/*
	 *	Bank, die gestartet werden soll.
	 *
	 */
	unsigned char	active_bank;

	/*
	 *	Informationen ueber die beiden Flash-Baenke
	 *
	 */
	struct Bank	bank[2];

	/*
	 *	CRC16 ueber die Boot-Daten
	 *
	 */
	unsigned int 	crc;
};


/*
 *	Struktur der Firmware-Informationen. Diese Infos muessen
 *	sich in einer Firmware befinden, die vom boot-Modul gestartet
 *	werden soll.
 *
 */
struct FWInfo {
	/*
	 *	Magic-Number anhand derer die korrekte Firmware
	 *	erkannt wird.
	 *
	 */
	unsigned int	magic;

	/*
	 *	Firmware-Version.
	 *
	 */
	unsigned char	version_major;
	unsigned char	version_minor;
	unsigned char	version_patch;
};

void boot_print_state (struct AX25_cb xdata *axcb);
char boot_check (unsigned char bank);
char boot_set_bank (unsigned char bank);
void boot_enable_bank (unsigned char bank, unsigned int crc);
void boot_disable_bank (unsigned char bank);
void boot ();

#endif

