
/*
 *	$Id: buf.c,v 1.1.1.1 2002/02/15 22:36:40 Klaus Exp $
 *
 *	buf.c		Einfache Pufferverwaltung.
 *
 *
 *
 *	(c) 1999,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "system.h"
#include "buf.h"
#include "mm.h"

/*---------------------------------------------------------------------------*/



/*
 *	Erzeugt einen neuen Puffer. Wenn der freie Speicher fuer
 *	den Puffer nicht ausreicht, wird NULL zurueckgegeben.
 *
 */
struct Buffer xdata * buf_new (unsigned int size)
{
	struct Buffer xdata * idata buf;

	if ( (buf = malloc(sizeof(struct Buffer) + size)) != NULL ) {
		buf->next = NULL;
		buf->head = buf->tail = (unsigned char xdata *)(buf + 1);
		buf->tail_end = buf->tail + size;

		buf->locked = 0;
	}

	return buf;
}



/*
 *	Loescht den Puffer *buf.
 *
 */
void buf_delete (struct Buffer xdata *buf)
{
	free(buf);
}



/*
 *	Gibt den Zeiger auf den Datenbereich des Puffers
 *	zurueck.
 *
 */
unsigned char xdata * buf_data (struct Buffer xdata *buf)
{
	return buf->head;
}



/*
 *	Gibt die Anzahl der Datenbytes, die in *buf abgelegt
 *	sind zurueck.
 *
 */
unsigned int buf_size (struct Buffer xdata *buf)
{
	return buf->tail - buf->head;
}


/*
 *	Gibt die Anzahl der Bytes zurueck, die noch an den Puffer
 *	<buf> angefuegt werden koennen
 *
 */
unsigned int buf_tailroom (struct Buffer xdata *buf)
{
	return buf->tail_end - buf->tail;
}


/*
 *	Fuegt die Daten <dat>/<datlen> an den Puffer <buf> an. Wenn
 *	nicht genuegend Speicher im Puffer <buf> frei ist, wird
 *	0 zurueckgegeben, ansonsten 1.
 *
 */
char buf_append (struct Buffer xdata *buf,unsigned char xdata *dat,
	          unsigned int datlen)
{
	unsigned char xdata *tail;

	tail = buf->tail;

	if (buf->tail_end - tail < datlen)
		return 0;

	while (datlen) {
		*tail++ = *dat++;
		datlen--;
	}

	buf->tail = tail;

	return 1;
}


/*
 *	Legt das Byte ch im Puffer *buf ab. Es wird die Anzahl
 *	der erfolgreich hinzugefuegten Bytes zurueckgegeben.
 *
 */
char buf_putch (struct Buffer xdata *buf,unsigned char ch)
{
	if (buf->tail == buf->tail_end)
		return 0;

	*buf->tail++ = ch;
	return 1;
}



/*
 *	Gibt das naechste Zeichen aus dem Puffer *buf zurueck.
 *	Ist der Puffer leer, so wird -1 zurueckgegeben.
 *
 */
int buf_getch (struct Buffer xdata *buf)
{
	unsigned char idata ch;

	if (buf->head == buf->tail)
		return -1;

	ch = *buf->head++;

	return (int)ch;
}


/*
 *	Funktion arbeitet genauso, wie buf_getch() nur mit dem
 *	Unterschied, dass das Byte am Ende des Puffer ausgelesen
 *	wird.
 *
 */
int buf_getch_tail (struct Buffer xdata *buf)
{
	if (buf->head == buf->tail)
		return -1;

	buf->tail--;

	return (int)((unsigned char)*buf->tail);
}


/*
 *	Holt len Bytes aus dem Puffer *buf und gibt den Zeiger
 *	Zeiger auf das Array aus den ausgelesenen Bytes zurueck.
 *	Wenn weniger als len Bytes im Puffer sind, wird NULL
 *	zurueckgegeben.
 *
 */
void xdata * buf_pull (struct Buffer xdata *buf,int len)
{
	unsigned int idata bufsize;
	void xdata * idata ptr;

	bufsize = buf->tail - buf->head;

	if (bufsize < len)
		return NULL;

	ptr = buf->head;
	buf->head += len;

	return ptr;
}


