
/*
 *	$Id: cmdint.c,v 1.2 2002/03/16 11:24:27 klaus Exp $
 *
 *	cmdint.c	Simpler Kommandointerpreter
 *
 *
 *			
 *	(c) 2000,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "cmdint.h"
#include "system.h"
#include "ax25.h"
#include "ax25_l1.h"
#include "mm.h"
#include "string.h"

/*
 *	Anzahl der Befehle pro Zeile bei der Kommandolistenausgabe
 *	cmd_showcmds()
 *
 */
#define CMDSPERLINE	5

 
/*---------------------------------------------------------------------------*/


/*
 *	Gibt alle Kommandos aus der Tabelle <cmdtab> formatiert
 *	auf <axcb> aus.
 *
 */
void cmd_showcmds (struct AX25_cb xdata *axcb,
                   struct Command code *cmdtab)
{
	unsigned char cnt;

	cnt = CMDSPERLINE - 1;

	while (cmdtab->id != CMD_ID_END) {

		if (cnt) {
			ax25_printf(axcb,"%-12S",(__CPTR__)cmdtab->cmd);
			cnt--;
		} else {
			ax25_printf(axcb,"%S\r",(__CPTR__)cmdtab->cmd);
			cnt = CMDSPERLINE - 1;
		}

		cmdtab++;
	}

	if (cnt != CMDSPERLINE - 1)
		ax25_printf(axcb,"\r");
}


/*
 *	Gibt die Gebrauchsanweisung fuer das Kommando
 *	<cmd> aus.
 *
 */
void cmd_show_usage (struct AX25_cb xdata *axcb,
		     struct Command code *cmd)
{
	ax25_printf(axcb, "USAGE: %S", (__CPTR__)cmd->cmd);

	if (cmd->parm_desc)
		ax25_printf(axcb, " %S\r", (__CPTR__)cmd->parm_desc);
	else
		ax25_printf(axcb, "\r");
}


/*---------------------------------------------------------------------------*/


/*
 *	Prueft, ob <cmd> eine Kommando-Approximation von
 *	<fullcmd> ist. Wenn dem so ist, wird 1 zurueckgegeben,
 *	ansonsten 0.
 *
 */
static char cmd_isapprox (char xdata *cmd,char code *fullcmd)
{
	while (*cmd && *cmd == *fullcmd) {
		cmd++;
		fullcmd++;
	}

	if (!*cmd)
		return 1;

	return 0;
}


/*---------------------------------------------------------------------------*/


/*
 *	Decodiert aus <parm> ein Rufzeichen incl. SSID. Das
 *	Ergebnis wird nach <argv->call> geschrieben. Im
 *	Fehlerfall wird -1 zurueckgegeben, ansonsten 0.
 *
 */
static char cmd_call (char xdata *parm, union ARGV xdata *argv)
{
	unsigned char n;

	for (n = 0; n < AX25_CALL_SIZE; n++) {
		char ch;
		
		if (!*parm || *parm == '-')
			break;

		ch = *parm;
		
		if (ch >= 'a' && ch <= 'z')
			ch -= 0x20;

		if (ch < 'A' || ch > 'Z')
			if (ch < '0' || ch > '9')
				return -1;
		
		argv->call.call[n] = ch << 1;

		parm++;
	}

	for (; n < AX25_CALL_SIZE; n++)
		argv->call.call[n] = ' ' << 1;

	if (*parm == '-') {
		unsigned long ssid;
		char xdata *endptr;
		
		ssid = strtoul(parm + 1, &endptr, 10);

		if (*endptr || ssid > 15)
			return -1;
		
		argv->call.ssid = (unsigned char)ssid << 1;
	} else if (!*parm) {
		argv->call.ssid = 0;
	} else
		return -1;

	return 0;	
}


/*
 *	Ermittelt den bool'schen Wert von <parm> und speichert
 *	ihn in <argv->boolean>. Es werden nur die beiden
 *	Schluesselwoerter "on" und "off" erkannt. Im Fehlerfall
 *	wird -1 zurueckgegeben, ansonsten 0.
 *
 */
static char cmd_bool (char xdata *parm, union ARGV xdata *argv)
{
	if (!strcmp_xc(parm, "on")) {
		argv->boolean = 1;
		return 0;
	}

	if (!strcmp_xc(parm, "off")) {
		argv->boolean = 0;
		return 0;
	}

	return -1;
}


/*
 *	Wandelt den Parameter <parm> in ein unsigned char, unsiged int
 *	oder unsigned long Wert um. In <size> muss angegeben werden
 *	wie gross der ziel-Datentyp (in Bytes) ist.
 *	Im Fehlerfall wird -1 zurueckgegeben, ansonsten 0.
 *
 */
static char cmd_num (char xdata *parm, union ARGV xdata *argv, char size)
{
	char xdata *endptr;
	unsigned long idata num;

	num = strtoul(parm, &endptr, 10);

	if (*endptr)
		return -1;

	switch (size) {
	case 1:
		if (num & 0xffffff00)
			return -1;

		argv->u8 = (unsigned char)num;

		break;
	case 2:
		if (num & 0xffff0000)
			return -1;

		argv->u16 = (unsigned int)num;
		break;

	default:
		argv->u32 = num;
	}

	return 0;
}


/*
 *	Wandelt den Bitspring in <parm> in einen unsigned int um.
 *	Im Fehlerfall wird -1 zurueckgegeben, ansonsten 0.
 *
 */
static char cmd_bstr (char xdata *parm, union ARGV xdata *argv)
{
	char xdata *endptr;

	argv->u16 = bstrtoui(parm, &endptr);

	if (*endptr)
		return -1;

	return 0;
}


/*
 *	Ermittelt den Kommandoeintrag, der zu dem Kommando-String
 *	<str> passt. Wenn kein passender Eintrag gefunden wurd, wird
 *	NULL zurueckgegeben.
 *
 *	Vor dem Aufruf der Funktion muss der Kommando-String mit
 *	strtok_init() gesetzt werden!
 *
 */
static struct Command code * cmd_lookup (struct Command code *cmdtab)
{
	struct Command code *cmd = NULL;
	char xdata *token;

	do {
		/*
		 *	Naechstes Token aus dem String
		 *	<str> holen. Wenn der String leer
		 *	ist, dann wird das bis dahin decodierte
		 *	Kommando zurueckgegeben.
		 *
		 */
		if ( !(token = strtok()) )
			return cmd;

		/* 
		 *	Zugehoerigen Eintrag in der Kommandotabelle suchen 
		 *
		 */
		for (cmd = cmdtab; cmd->id != CMD_ID_END; cmd++)
			if (cmd_isapprox(token, cmd->cmd))
				break;
		
		if (cmd->id == CMD_ID_END)
			return NULL;

		/*
		 *	Nun muss in der Sub-Kommandotabelle
		 *	weitergesucht werden.
		 *
		 */
		cmdtab = cmd->subtab;
			
		/*
		 *	Wenn keine Subkommandotabelle existiert,
		 *	sind wir fertig.
		 *
		 */
	} while (cmdtab);

	return cmd;
}


/*
 *	Interpretiert den Kommando-String und gibt den Zeiger
 *	auf die Argument-Tabelle zurueck.
 *	Wenn ein Fehler aufgetreten ist, wird eine Meldung ausgegeben
 *	und NULL zurueckgegeben. Wenn kein Kommando erkannt wurd (z.B.
 *	bei einer leeren Zeile) wird NULL zurueckegeben.
 *
 */
struct Args xdata * cmd_interprete (struct AX25_cb xdata *axcb,
		                    char xdata *cmdstr,
                                    struct Command code *cmdtab)
{
	static struct Args xdata args;
	struct Command code *cmd;
	char xdata *token;
	char code *parm;
	char optional,error;
	unsigned char argc;
	union ARGV xdata *argv;
	
	/*
	 *	Leere Kommandozeilen werden ignoriert.
	 *
	 */
	strtok_init(cmdstr, " \t", 1);
	
	if (!strtok_tail())
		return NULL;
	
	if ( !(cmd = cmd_lookup(cmdtab)) ) {
		ax25_printf(axcb, "invalid command\r");
		return NULL;
	}

	/*
	 *	Wenn nur der Name einer Kommandogruppe eingegeben
	 *	wurde, dann werden alle Unterkommandos angezeigt.
	 *	Quick-Hack, damit die Gruppennamen nicht als
	 *	Fehler interpretiert werden. Muss mal ueberlegen,
	 *	wie ich das eleganter loesen kann...
	 *
	 */
	if (cmd->subtab) {
		ax25_printf(axcb, "Subcommands:\r");
		cmd_showcmds(axcb, cmd->subtab);
		return NULL;	
	}
	
	args.id = cmd->id;

	parm = cmd->parm;
	optional = 0;
	error = 0;
	
	argc = 0;
	argv = args.argv;
	
	while (parm && *parm && !error) {
		/*
		 *	Alle Parameter nach '|' sind optional!
		 *
		 */
		if (*parm == '|') {
			optional = 1;
			parm++;
			continue;
		}
		
		/*
		 *	'T' steht fuer "tail" und bedeutet, dass 
		 *	der Zeiger args.tail auf den noch nicht
		 *	interpretierten Rest des Kommandos gesetzt wird.
		 *	Wenn kein Rest mehr existiert, wird args.tail auf
		 *	NULL gesetzt (s. strtok_tail()).
		 *	Alle weiteren Parameter nach 't' werden
		 *	ignoriert!
		 *
		 */
		if (*parm == 'T') {
			args.tail = strtok_tail();
			break;
		}

		/*
		 *	Wertet den Rest des Strings als Kommando
		 * 	aus und setzt args.cmd auf den ermittelten
		 *	Kommando-Eintrag.
		 *
		 */
		if (*parm == 'C') {
			args.cmd = cmd_lookup(cmdtab);
			args.tail = strtok_tail();
			break;
		}
	
		/*
		 *	Falls vergessen wurde in cmdint.h ARGS_MAX
		 *	anzupassen, wird hier zur Sicherheit eine
		 *	Fehlermeldung generiert.
		 *
		 */
		if (argc >= ARGS_MAX) {
			ax25_printf(axcb, "cmdint: argv[] overflow\r");
			return NULL;
		}

		/*
		 *	Naechstes Token von der Zeile abtrennen.
		 *
		 */
		if ( !(token = strtok()) ) {
			if (optional)
				break;

			ax25_printf(axcb,"too few arguments\r");
			return NULL;
		}

		/*
		 *	Token je nach gefordertem Parameter-Typ
		 *	auswerten.
		 *
		 */
		switch (*parm) {
		case 'c':
			error = cmd_call(token, argv);
			break;
			
		case 'y':
			error = cmd_num(token, argv, 1);
			break;
			
		case 'u':
			error = cmd_num(token, argv, 2);
			break;
	
		case 'l': 
			error = cmd_num(token, argv, 4);
			break;

		case 't':
			error = cmd_bstr(token, argv);
			break;

		case 'b':
			error = cmd_bool(token, argv);
			break;

		case 's':
			argv->string = token;
			break;
		
		default:
			ax25_printf(axcb,"internal error\r");
			return NULL;
		}

		parm++;

		argc++;
		argv++;
	}

	if (error) {
		ax25_printf(axcb,"invalid parameter >%s<\r",(__XPTR__)token);
		return NULL;
	}

	if ((!parm || !*parm) && strtok()) {
		ax25_printf(axcb, "too many arguments\r");
		return NULL;
	}

	args.argc = argc;

	return &args;
}

 
