
#ifndef _CONFIG_H_
#define _CONFIG_H_

#include "ax25_l1.h"
#include "master.h"
#include "repeater.h"


/*
 *	Adresse, ab der die Konfigurationsdaten im EEPROM
 *	beginnen.
 *
 */
#define CONF_ADDR	0x0040

/*
 *	Magic-Number
 *
 */
#define CONF_MAGIC	0xafb0	


/*
 *	Speicherlayout der Konfigurationsdaten im
 *	EEPROM.
 *
 */
struct Config {
	/*
	 *	Magic-Number. Anhand der die 
	 *	Konfigurationsdaten erkannt werden koennen.
	 *
	 */
	unsigned int		magic;

	/*
	 *	Die wichtigsten Systemdaten werden in einer eigenen
	 *	Struktur gehalten. Das System versucht immer diese
	 *	Daten beizubehalten, auch wenn sich das Speicher-
	 *	Layout der Konfiguration aendert. Das setzt voraus,
	 *	dass sich diese Daten immer an der selben Stelle im
	 *	EEPROM befinden muessen und nicht modifiziert werden.
	 *
	 *	Wenn bei einem RPC die Firmware im EPROM
	 *	ein anderes Konfigurations-Speicherlayout besitzt,
	 *	als die Firmware in einer Flash-Bank, so ist das
	 *	System trotzdem zu jeder Zeit mit dem Sysop-Passwort
	 *	geschuetzt.
	 *
	 */
	struct {
		char			passwd[81];	/* Sysop Passwort */
		struct AX25_address	mycall;		/* RPC MyCall */

		unsigned char		reserved[32];
		
		unsigned int		crc16;	
		
	} hot_data;


	/*
	 *	Allgemeine Konfigurationsdaten
	 *
	 */
	char			ctext[201];
	unsigned long		ident_addr;
	char			ident_text[51];

	/*
	 *	Konfigurationsdaten des AX.25-Protokollstacks.
	 *	Das MyCall ist im geschuetzten Bereich
	 *	untergebracht!
	 *
	 */
	unsigned char 		l2_unack;
	unsigned char 		l2_n2;
	unsigned char 		l2_t1;
	unsigned char 		l2_t2;
	unsigned char 		l2_t3;

	/*
	 *	Konfigurationsdaten des SCC
	 *
	 */
	unsigned char		scc_txinvert : 1;
	unsigned char		scc_rxinvert : 1;
	unsigned char 		scc_txhead;
	unsigned char 		scc_txtail;
	unsigned int		scc_preamble;

	/*
	 *	Master-Tabelle
	 *
	 */
	struct Master		mtab[MASTER_MAX];

	/*
	 *	Konfigurationsdaten des Repeaters
	 *
	 */
	struct Trigger		rp_ttab[TRIGGER_MAX];

	/*
	 *	Am Ende der Daten steht die CRC (CRC16)
	 *	Die CRC wird ueber alle Daten (ausser der CRC)
	 *	berechnet.
	 *
	 */
	unsigned int 		crc16;
};

extern struct Config xdata conf;

void conf_init ();
void conf_save ();

#endif


