
/*
 *	$Id: dectabs.c,v 1.5 2002/05/18 11:41:48 Klaus Exp $
 *
 *	dectabs.c	SCC-Decoder-Tabellen
 *
 *
 *
 *
 *	(c) 2000,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "dectabs.h"
#include "scc.h"

/*--------------------------------------------------------------------------*/


/*
 *	Auto-Sensing-Tabelle
 *
 *	Mit dieser Tabelle fuert der Receiver im SCC-Modul das
 *	Datenraten-Autosensing durch.
 *	Wenn die Praeambel einer Funkrufaussendung empfangen wird
 *	und diese Tabelle mit der gemessenen Bitlaenge indiziert
 *	wird, kann ueber den Tabellenwert ermittelt werden, um
 *	welche Datenrate es sich handelt.
 *
 *	Das Bit7 zeigt an, ob sich der Flankenwechsel im
 *	Toleranzbereich befindet.
 *	
 *
 */
unsigned char code astab[64] =
{
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 
	
	0x80 | SCC_B2400, 
	0x80 | SCC_B2400, 
	0x80 | SCC_B2400, 
	
	
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	
	0x81 | SCC_B1200, 
	0x81 | SCC_B1200, 
	0x81 | SCC_B1200, 
	0x81 | SCC_B1200, 
	0x81 | SCC_B1200, 
	0x81 | SCC_B1200, 
	
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512, 
	0x80 | SCC_B512
};


/*--------------------------------------------------------------------------*/


/*
 *	Die folgenden Tabellen werden im SCC-Modul zum Decodieren
 *	des RX-Signals verwendet.
 *
 *	Tabelle, mit der beim Receiver die Dauer zwischen zwei
 *	Flankenwechseln in die Anzahl der empfangenen Bits 
 *	umgerechnet wird. Das High-Bit (Bit7) zeigt an, ob
 *	sich das Timing des Flankenwechsels im Toleranzbereich
 *	befindet. Diese Information wird benutzt, um das
 *	DCD-Signal zu erzeugen.
 *
 *	Aufbau eines Eintrags:
 *
 *      +---+---+---+---+---+---+---+---+
 *	| 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
 *      +---+---+---+---+---+---+---+---+
 *
 *	  ^   \_______________________/
 *        |               |
 *        |               `----------------- Anzahl empfangene Bits
 *        |
 *        `--------------------------------- roh-DCD
 *					     1 => Flankenwechsel im
 *						  Toleranzbereich
 *
 *
 */
 
static unsigned char code dectab_512[1828] =
{
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 

	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x01, 0x01, 0x01, 0x01, 0x81, 0x81, 0x81, 
	0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 
	0x81, 0x81, 0x81, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 

	0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 
	0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 
	0x02, 0x02, 0x02, 0x02, 0x02, 0x82, 0x82, 0x82, 
	0x82, 0x82, 0x82, 0x82, 0x82, 0x82, 0x82, 0x82, 
	0x82, 0x82, 0x82, 0x02, 0x02, 0x02, 0x02, 0x02, 
	0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 
	0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 

	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x03, 0x83, 0x83, 0x83, 
	0x83, 0x83, 0x83, 0x83, 0x83, 0x83, 0x83, 0x83, 
	0x83, 0x83, 0x83, 0x03, 0x03, 0x03, 0x03, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 

	0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 
	0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 
	0x04, 0x04, 0x04, 0x04, 0x04, 0x84, 0x84, 0x84, 
	0x84, 0x84, 0x84, 0x84, 0x84, 0x84, 0x84, 0x84, 
	0x84, 0x84, 0x84, 0x84, 0x04, 0x04, 0x04, 0x04, 
	0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 
	0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 
	0x04, 

	0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 
	0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 
	0x05, 0x05, 0x05, 0x05, 0x05, 0x85, 0x85, 0x85, 
	0x85, 0x85, 0x85, 0x85, 0x85, 0x85, 0x85, 0x85, 
	0x85, 0x85, 0x85, 0x05, 0x05, 0x05, 0x05, 0x05, 
	0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 
	0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 

	0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 
	0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 
	0x06, 0x06, 0x06, 0x06, 0x06, 0x86, 0x86, 0x86, 
	0x86, 0x86, 0x86, 0x86, 0x86, 0x86, 0x86, 0x86, 
	0x86, 0x86, 0x86, 0x06, 0x06, 0x06, 0x06, 0x06, 
	0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 
	0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 

	0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
	0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
	0x07, 0x07, 0x07, 0x07, 0x07, 0x87, 0x87, 0x87, 
	0x87, 0x87, 0x87, 0x87, 0x87, 0x87, 0x87, 0x87, 
	0x87, 0x87, 0x87, 0x07, 0x07, 0x07, 0x07, 0x07, 
	0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
	0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 

	0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
	0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
	0x08, 0x08, 0x08, 0x08, 0x08, 0x88, 0x88, 0x88, 
	0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 
	0x88, 0x88, 0x88, 0x88, 0x08, 0x08, 0x08, 0x08, 
	0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
	0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
	0x08, 

	0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 
	0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 
	0x09, 0x09, 0x09, 0x09, 0x09, 0x89, 0x89, 0x89, 
	0x89, 0x89, 0x89, 0x89, 0x89, 0x89, 0x89, 0x89, 
	0x89, 0x89, 0x89, 0x09, 0x09, 0x09, 0x09, 0x09, 
	0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 
	0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 

	0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 
	0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 
	0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x8a, 0x8a, 0x8a, 
	0x8a, 0x8a, 0x8a, 0x8a, 0x8a, 0x8a, 0x8a, 0x8a, 
	0x8a, 0x8a, 0x8a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 
	0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 
	0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 

	0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 
	0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 
	0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x8b, 0x8b, 0x8b, 
	0x8b, 0x8b, 0x8b, 0x8b, 0x8b, 0x8b, 0x8b, 0x8b, 
	0x8b, 0x8b, 0x8b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 
	0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 
	0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 

	0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 
	0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 
	0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x8c, 0x8c, 0x8c, 
	0x8c, 0x8c, 0x8c, 0x8c, 0x8c, 0x8c, 0x8c, 0x8c, 
	0x8c, 0x8c, 0x8c, 0x8c, 0x0c, 0x0c, 0x0c, 0x0c, 
	0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 
	0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 
	0x0c, 

	0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 
	0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 
	0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x8d, 0x8d, 0x8d, 
	0x8d, 0x8d, 0x8d, 0x8d, 0x8d, 0x8d, 0x8d, 0x8d, 
	0x8d, 0x8d, 0x8d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 
	0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 
	0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 

	0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 
	0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 
	0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x8e, 0x8e, 0x8e, 
	0x8e, 0x8e, 0x8e, 0x8e, 0x8e, 0x8e, 0x8e, 0x8e, 
	0x8e, 0x8e, 0x8e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 
	0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 
	0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 

	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x8f, 0x8f, 0x8f, 
	0x8f, 0x8f, 0x8f, 0x8f, 0x8f, 0x8f, 0x8f, 0x8f, 
	0x8f, 0x8f, 0x8f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 

	0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 
	0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 
	0x10, 0x10, 0x10, 0x10, 0x10, 0x90, 0x90, 0x90, 
	0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 
	0x90, 0x90, 0x90, 0x90, 0x10, 0x10, 0x10, 0x10, 
	0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 
	0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 
	0x10, 

	0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 
	0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 
	0x11, 0x11, 0x11, 0x11, 0x11, 0x91, 0x91, 0x91, 
	0x91, 0x91, 0x91, 0x91, 0x91, 0x91, 0x91, 0x91, 
	0x91, 0x91, 0x91, 0x11, 0x11, 0x11, 0x11, 0x11, 
	0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 
	0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 

	0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 
	0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 
	0x12, 0x12, 0x12, 0x12, 0x12, 0x92, 0x92, 0x92, 
	0x92, 0x92, 0x92, 0x92, 0x92, 0x92, 0x92, 0x92, 
	0x92, 0x92, 0x92, 0x12, 0x12, 0x12, 0x12, 0x12, 
	0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 
	0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 

	0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 
	0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 
	0x13, 0x13, 0x13, 0x13, 0x13, 0x93, 0x93, 0x93, 
	0x93, 0x93, 0x93, 0x93, 0x93, 0x93, 0x93, 0x93, 
	0x93, 0x93, 0x93, 0x13, 0x13, 0x13, 0x13, 0x13, 
	0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 
	0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 

	0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 
	0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 
	0x14, 0x14, 0x14, 0x14, 0x14, 0x94, 0x94, 0x94, 
	0x94, 0x94, 0x94, 0x94, 0x94, 0x94, 0x94, 0x94, 
	0x94, 0x94, 0x94, 0x94, 0x14, 0x14, 0x14, 0x14, 
	0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 
	0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 
	0x14, 

	0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 
	0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 
	0x15, 0x15, 0x15, 0x15, 0x15, 0x95, 0x95, 0x95, 
	0x95, 0x95, 0x95, 0x95, 0x95, 0x95, 0x95, 0x95, 
	0x95, 0x95, 0x95, 0x15, 0x15, 0x15, 0x15, 0x15, 
	0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 
	0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 

	0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 
	0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 
	0x16, 0x16, 0x16, 0x16, 0x16, 0x96, 0x96, 0x96, 
	0x96, 0x96, 0x96, 0x96, 0x96, 0x96, 0x96, 0x96, 
	0x96, 0x96, 0x96, 0x16, 0x16, 0x16, 0x16, 0x16, 
	0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 
	0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 

	0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 
	0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 
	0x17, 0x17, 0x17, 0x17, 0x17, 0x97, 0x97, 0x97, 
	0x97, 0x97, 0x97, 0x97, 0x97, 0x97, 0x97, 0x97, 
	0x97, 0x97, 0x97, 0x17, 0x17, 0x17, 0x17, 0x17, 
	0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 
	0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 

	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 
	0x18, 0x18, 0x18, 0x18, 0x18, 0x98, 0x98, 0x98, 
	0x98, 0x98, 0x98, 0x98, 0x98, 0x98, 0x98, 0x98, 
	0x98, 0x98, 0x98, 0x98, 0x18, 0x18, 0x18, 0x18, 
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 
	0x18, 

	0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 
	0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 
	0x19, 0x19, 0x19, 0x19, 0x19, 0x99, 0x99, 0x99, 
	0x99, 0x99, 0x99, 0x99, 0x99, 0x99, 0x99, 0x99, 
	0x99, 0x99, 0x99, 0x19, 0x19, 0x19, 0x19, 0x19, 
	0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 
	0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 

	0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 
	0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 
	0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x9a, 0x9a, 0x9a, 
	0x9a, 0x9a, 0x9a, 0x9a, 0x9a, 0x9a, 0x9a, 0x9a, 
	0x9a, 0x9a, 0x9a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 
	0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 
	0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 

	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x9b, 0x9b, 0x9b, 
	0x9b, 0x9b, 0x9b, 0x9b, 0x9b, 0x9b, 0x9b, 0x9b, 
	0x9b, 0x9b, 0x9b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 

	0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 
	0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 
	0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x9c, 0x9c, 0x9c, 
	0x9c, 0x9c, 0x9c, 0x9c, 0x9c, 0x9c, 0x9c, 0x9c, 
	0x9c, 0x9c, 0x9c, 0x9c, 0x1c, 0x1c, 0x1c, 0x1c, 
	0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 
	0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 
	0x1c, 

	0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 
	0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 
	0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x9d, 0x9d, 0x9d, 
	0x9d, 0x9d, 0x9d, 0x9d, 0x9d, 0x9d, 0x9d, 0x9d, 
	0x9d, 0x9d, 0x9d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 
	0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 
	0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 

	0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 
	0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 
	0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x9e, 0x9e, 0x9e, 
	0x9e, 0x9e, 0x9e, 0x9e, 0x9e, 0x9e, 0x9e, 0x9e, 
	0x9e, 0x9e, 0x9e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 
	0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 
	0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 

	0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 
	0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 
	0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x9f, 0x9f, 0x9f, 
	0x9f, 0x9f, 0x9f, 0x9f, 0x9f, 0x9f, 0x9f, 0x9f, 
	0x9f, 0x9f, 0x9f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 
	0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 
	0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 

	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 
	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 
	0x20, 0x20, 0x20, 0x20, 0x20, 0xa0, 0xa0, 0xa0, 
	0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 
	0xa0, 0xa0, 0xa0, 0xa0, 0x20, 0x20, 0x20, 0x20, 
	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 
	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 
};


static unsigned char code dectab_1200[780] =
{
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 

	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x01, 
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 

	0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 
	0x02, 0x82, 0x82, 0x82, 0x82, 0x82, 0x82, 0x02, 
	0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 

	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 
	0x03, 0x83, 0x83, 0x83, 0x83, 0x83, 0x83, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 

	0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 
	0x04, 0x84, 0x84, 0x84, 0x84, 0x84, 0x84, 0x04, 
	0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 

	0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 
	0x05, 0x85, 0x85, 0x85, 0x85, 0x85, 0x85, 0x05, 
	0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 

	0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 
	0x06, 0x86, 0x86, 0x86, 0x86, 0x86, 0x86, 0x06, 
	0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 

	0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
	0x07, 0x87, 0x87, 0x87, 0x87, 0x87, 0x87, 0x07, 
	0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 

	0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
	0x08, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x08, 
	0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 

	0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 
	0x09, 0x89, 0x89, 0x89, 0x89, 0x89, 0x89, 0x09, 
	0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 

	0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 
	0x0a, 0x8a, 0x8a, 0x8a, 0x8a, 0x8a, 0x8a, 0x0a, 
	0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 

	0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 
	0x0b, 0x8b, 0x8b, 0x8b, 0x8b, 0x8b, 0x8b, 0x0b, 
	0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 

	0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 
	0x0c, 0x8c, 0x8c, 0x8c, 0x8c, 0x8c, 0x8c, 0x0c, 
	0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 

	0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 
	0x0d, 0x8d, 0x8d, 0x8d, 0x8d, 0x8d, 0x8d, 0x0d, 
	0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 

	0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 
	0x0e, 0x8e, 0x8e, 0x8e, 0x8e, 0x8e, 0x8e, 0x0e, 
	0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 

	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x8f, 0x8f, 0x8f, 0x8f, 0x8f, 0x8f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 

	0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 
	0x10, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x10, 
	0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 

	0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 
	0x11, 0x91, 0x91, 0x91, 0x91, 0x91, 0x91, 0x11, 
	0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 

	0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 
	0x12, 0x92, 0x92, 0x92, 0x92, 0x92, 0x92, 0x12, 
	0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 

	0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 
	0x13, 0x93, 0x93, 0x93, 0x93, 0x93, 0x93, 0x13, 
	0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 0x13, 

	0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 
	0x14, 0x94, 0x94, 0x94, 0x94, 0x94, 0x94, 0x14, 
	0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 0x14, 

	0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 
	0x15, 0x95, 0x95, 0x95, 0x95, 0x95, 0x95, 0x15, 
	0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 0x15, 

	0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 
	0x16, 0x96, 0x96, 0x96, 0x96, 0x96, 0x96, 0x16, 
	0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 0x16, 

	0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 
	0x17, 0x97, 0x97, 0x97, 0x97, 0x97, 0x97, 0x17, 
	0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 

	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 
	0x18, 0x98, 0x98, 0x98, 0x98, 0x98, 0x98, 0x18, 
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 

	0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 
	0x19, 0x99, 0x99, 0x99, 0x99, 0x99, 0x99, 0x19, 
	0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 0x19, 

	0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 
	0x1a, 0x9a, 0x9a, 0x9a, 0x9a, 0x9a, 0x9a, 0x1a, 
	0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 

	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 
	0x1b, 0x9b, 0x9b, 0x9b, 0x9b, 0x9b, 0x9b, 0x1b, 
	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 

	0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 
	0x1c, 0x9c, 0x9c, 0x9c, 0x9c, 0x9c, 0x9c, 0x1c, 
	0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 

	0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 
	0x1d, 0x9d, 0x9d, 0x9d, 0x9d, 0x9d, 0x9d, 0x1d, 
	0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 

	0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 
	0x1e, 0x9e, 0x9e, 0x9e, 0x9e, 0x9e, 0x9e, 0x1e, 
	0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 

	0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 
	0x1f, 0x9f, 0x9f, 0x9f, 0x9f, 0x9f, 0x9f, 0x1f, 
	0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 

	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 
	0x20, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0xa0, 0x20, 
	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20 
};


static unsigned char code dectab_2400[390] =
{
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 

	0x01, 0x01, 0x01, 0x01, 0x01, 0x81, 0x81, 0x81, 
	0x01, 0x01, 0x01, 0x01, 

	0x02, 0x02, 0x02, 0x02, 0x02, 0x82, 0x82, 0x82, 
	0x02, 0x02, 0x02, 0x02, 

	0x03, 0x03, 0x03, 0x03, 0x03, 0x83, 0x83, 0x83, 
	0x03, 0x03, 0x03, 0x03, 

	0x04, 0x04, 0x04, 0x04, 0x04, 0x84, 0x84, 0x84, 
	0x04, 0x04, 0x04, 0x04, 

	0x05, 0x05, 0x05, 0x05, 0x05, 0x85, 0x85, 0x85, 
	0x05, 0x05, 0x05, 0x05, 

	0x06, 0x06, 0x06, 0x06, 0x06, 0x86, 0x86, 0x86, 
	0x06, 0x06, 0x06, 0x06, 

	0x07, 0x07, 0x07, 0x07, 0x07, 0x87, 0x87, 0x87, 
	0x07, 0x07, 0x07, 0x07, 

	0x08, 0x08, 0x08, 0x08, 0x08, 0x88, 0x88, 0x88, 
	0x08, 0x08, 0x08, 0x08, 

	0x09, 0x09, 0x09, 0x09, 0x09, 0x89, 0x89, 0x89, 
	0x09, 0x09, 0x09, 0x09, 

	0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x8a, 0x8a, 0x8a, 
	0x0a, 0x0a, 0x0a, 0x0a, 

	0x0b, 0x0b, 0x0b, 0x0b, 0x0b, 0x8b, 0x8b, 0x8b, 
	0x0b, 0x0b, 0x0b, 0x0b, 

	0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x8c, 0x8c, 0x8c, 
	0x0c, 0x0c, 0x0c, 0x0c, 

	0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x8d, 0x8d, 0x8d, 
	0x0d, 0x0d, 0x0d, 0x0d, 

	0x0e, 0x0e, 0x0e, 0x0e, 0x0e, 0x8e, 0x8e, 0x8e, 
	0x0e, 0x0e, 0x0e, 0x0e, 

	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x8f, 0x8f, 0x8f, 
	0x0f, 0x0f, 0x0f, 0x0f, 

	0x10, 0x10, 0x10, 0x10, 0x10, 0x90, 0x90, 0x90, 
	0x10, 0x10, 0x10, 0x10, 

	0x11, 0x11, 0x11, 0x11, 0x11, 0x91, 0x91, 0x91, 
	0x11, 0x11, 0x11, 0x11, 

	0x12, 0x12, 0x12, 0x12, 0x12, 0x92, 0x92, 0x92, 
	0x12, 0x12, 0x12, 0x12, 

	0x13, 0x13, 0x13, 0x13, 0x13, 0x93, 0x93, 0x93, 
	0x13, 0x13, 0x13, 0x13, 

	0x14, 0x14, 0x14, 0x14, 0x14, 0x94, 0x94, 0x94, 
	0x14, 0x14, 0x14, 0x14, 

	0x15, 0x15, 0x15, 0x15, 0x15, 0x95, 0x95, 0x95, 
	0x15, 0x15, 0x15, 0x15, 

	0x16, 0x16, 0x16, 0x16, 0x16, 0x96, 0x96, 0x96, 
	0x16, 0x16, 0x16, 0x16, 

	0x17, 0x17, 0x17, 0x17, 0x17, 0x97, 0x97, 0x97, 
	0x17, 0x17, 0x17, 0x17, 

	0x18, 0x18, 0x18, 0x18, 0x18, 0x98, 0x98, 0x98, 
	0x18, 0x18, 0x18, 0x18, 

	0x19, 0x19, 0x19, 0x19, 0x19, 0x99, 0x99, 0x99, 
	0x19, 0x19, 0x19, 0x19, 

	0x1a, 0x1a, 0x1a, 0x1a, 0x1a, 0x9a, 0x9a, 0x9a, 
	0x1a, 0x1a, 0x1a, 0x1a, 

	0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x9b, 0x9b, 0x9b, 
	0x1b, 0x1b, 0x1b, 0x1b, 

	0x1c, 0x1c, 0x1c, 0x1c, 0x1c, 0x9c, 0x9c, 0x9c, 
	0x1c, 0x1c, 0x1c, 0x1c, 

	0x1d, 0x1d, 0x1d, 0x1d, 0x1d, 0x9d, 0x9d, 0x9d, 
	0x1d, 0x1d, 0x1d, 0x1d, 

	0x1e, 0x1e, 0x1e, 0x1e, 0x1e, 0x9e, 0x9e, 0x9e, 
	0x1e, 0x1e, 0x1e, 0x1e, 

	0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x9f, 0x9f, 0x9f, 
	0x1f, 0x1f, 0x1f, 0x1f, 

	0x20, 0x20, 0x20, 0x20, 0x20, 0xa0, 0xa0, 0xa0, 
	0x20, 0x20, 0x20, 0x20 
};


/*--------------------------------------------------------------------------*/


/*
 *	Diese Decodertabelle kann mit einer Geschwindigkeitsangabe
 *	(SCC_B512, SCC_B1200, ...) indiziert werden, um alle decoder-
 *	abhaengigen Werte zu bekommen.
 *
 */
struct DecTab code dectab[SCC_BCNT] = {
	{
		SCC_B512,
		dectab_512,
		sizeof(dectab_512),
		SCC_TC_512_75
	},
	{
		SCC_B1200,
		dectab_1200,
		sizeof(dectab_1200),
		SCC_TC_1200_75
	},
	{
		SCC_B2400,
		dectab_2400,
		sizeof(dectab_2400),
		SCC_TC_2400_75
	}
};





