
#ifndef _FLASH_H_
#define _FLASH_H_

#include "system.h"

/*
 *	Unterstuetzte Flashes
 *
 */
#define FLASH_UNTESTED	0x00
#define FLASH_AM29F010	0x01
#define FLASH_AT29C010A 0x02
#define FLASH_UNKNOWN	0xff	

/*
 *	Die beiden Speicherbaenke
 *
 *	bank0:	0x00000 - 0x0ffff
 *	bank1:  0x10000 - 0x1ffff
 *
 */
#define FLASH_BANK0	0
#define FLASH_BANK1	1

/*
 *	Gibt die aktive Flash-Bank zurueck.
 *
 *	ACHTUNG: Aktiv bedeutet nicht eingeblendet!!!!
 *		 Es muss zusaetzlich mit IS_CSEG_FLASH()
 *		 geprueft werden!!
 *
 */
#define FLASH_ACTIVE_BANK()	(FLASH_A16_STATUS() ? 1 : 0)

/*
 *	Aktiviert die angegebene Flash-Bank. Sie wird
 *	nicht eingeblendet!
 *
 */
#define FLASH_ACTIVATE(bank)	FLASH_A16(bank)	


extern unsigned char flash_type;
extern unsigned char flash_manufacturer;
extern unsigned char flash_device_code;


void flash_init ();

void flash_erase (unsigned char bank);

void flash_write (unsigned char bank, 
		  unsigned int addr, 
                  void xdata *buf, 
		  unsigned int buflen);
		  
void flash_read (unsigned char bank,
		 unsigned int addr,
		 void xdata *buf,
		 unsigned int buflen);


#endif


