
/*
 *	$Id: infobox.c,v 1.21 2002/05/12 12:35:04 Klaus Exp $
 *
 *	infobox.c	Die Infobox ist ein Kommando-Interpreter, der
 *			dem normalen User Informationen ueber den
 *			Funkrufsender bietet. Der Sysop kann ferner
 *			Systemeinstellungen vornehmen und Firmware-Uploads
 *			durchfuehren.
 *
 *	
 *	(c) 2000,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "infobox.h"
#include "version.h"
#include "system.h"
#include "ax25.h"
#include "ax25_sup.h"
#include "mm.h"
#include "cmdint.h"
#include "config.h"
#include "string.h"
#include "rtc.h"
#include "sched.h"
#include "scc.h"
#include "pocsag.h"
#include "master.h"
#include "scp.h"
#include "passwd.h"
#include "input.h"
#include "upload.h"
#include "boot.h"
#include "flash.h"
#include "repeater.h"
#include "trace.h"

/*
 *	Kommando-Prompt
 *
 */
#define IBOX_PROMPT	"=>"

/*
 *	Kommando-ID-Definitionen
 *
 */
#define CMD_SYSOP		0x80	/* Sysop-Kommando */
#define CMD_GROUP_MASK		0x70

#define CMD_ROOT		0x00
#define CMD_BOOTDEV		0x01 | CMD_SYSOP
#define CMD_CTEXT		0x02 | CMD_SYSOP
#define CMD_FLASH		0x03 | CMD_SYSOP
#define CMD_HELP		0x04
#define CMD_IDENTITY		0x05 | CMD_SYSOP
#define CMD_PAGE		0x06 | CMD_SYSOP
#define CMD_PARMS		0x07
#define CMD_PASSWD		0x08 | CMD_SYSOP
#define CMD_QUIT		0x09
#define CMD_RESET		0x0a | CMD_SYSOP
#define CMD_STATE		0x0b
#define CMD_SYS			0x0c
#define CMD_TRACE		0x0d | CMD_SYSOP
#define CMD_USAGE		0x0e
#define CMD_WRITE		0x0f | CMD_SYSOP

#define CMD_AX25		0x10
#define CMD_AX25_MYCALL		0x11 | CMD_SYSOP
#define	CMD_AX25_N2		0x12 | CMD_SYSOP
#define CMD_AX25_PARMS		0x13 
#define CMD_AX25_T1		0x14 | CMD_SYSOP
#define CMD_AX25_T2		0x15 | CMD_SYSOP
#define CMD_AX25_T3		0x16 | CMD_SYSOP
#define CMD_AX25_UNACK		0x17 | CMD_SYSOP

#define CMD_SCC			0x20
#define CMD_SCC_CALIBRATE	0x21 | CMD_SYSOP
#define CMD_SCC_PARMS		0x22 
#define CMD_SCC_PREAMBLE	0x23 | CMD_SYSOP
#define CMD_SCC_RXINVERT	0x24 | CMD_SYSOP
#define CMD_SCC_TXHEAD		0x25 | CMD_SYSOP
#define CMD_SCC_TXTAIL		0x26 | CMD_SYSOP
#define CMD_SCC_TXINVERT	0x27 | CMD_SYSOP

#define CMD_MASTER		0x30
#define CMD_MASTER_ADD		0x31 | CMD_SYSOP
#define CMD_MASTER_CMODE	0x32 | CMD_SYSOP
#define CMD_MASTER_DEL		0x33 | CMD_SYSOP
#define CMD_MASTER_LIST		0x34
#define CMD_MASTER_PASSWD	0x35 | CMD_SYSOP

#define CMD_SCHED		0x40
#define CMD_SCHED_LOCK		0x41 | CMD_SYSOP
#define CMD_SCHED_STATE		0x42
#define CMD_SCHED_TIME		0x43
#define CMD_SCHED_QUEUE		0x44
#define CMD_SCHED_UNLOCK   	0x45 | CMD_SYSOP

#define CMD_TRIGGER 		0x60
#define CMD_TRIGGER_ADD		0x61 | CMD_SYSOP
#define CMD_TRIGGER_DEL		0x62 | CMD_SYSOP
#define CMD_TRIGGER_LIST	0x63


/*
 *	Kommando-Tabellen
 *
 */
static struct Command code cmdtab_ax25[] = {
	{ CMD_AX25_MYCALL,	"mycall", "c",	"<call>",	NULL	},
	{ CMD_AX25_N2,		"n2",	  "y",	"<0..255>",	NULL	},
	{ CMD_AX25_PARMS,	"parms",  NULL,	NULL,		NULL	},
	{ CMD_AX25_T1,		"t1",	  "y",	"<s>",		NULL	},
	{ CMD_AX25_T2,		"t2",	  "y",	"<0.1s>",	NULL	},
	{ CMD_AX25_T3,		"t3",	  "y",	"<10s>",	NULL	},
	{ CMD_AX25_UNACK,	"unack",  "y",	"<1..7>",	NULL	},

	CMDTAB_END
};

static struct Command code cmdtab_scc[] = {
	{ CMD_SCC_CALIBRATE, 	"calibrate", "u|y", "<bps> [sec]", NULL	},
	{ CMD_SCC_PARMS,	"parms",     NULL,  NULL,	   NULL	},
	{ CMD_SCC_PREAMBLE,  	"preamble",  "u",   "<bits>",      NULL	},
	{ CMD_SCC_RXINVERT,  	"rxinvert",  "b",   "<on|off>",    NULL	},
	{ CMD_SCC_TXHEAD,	"txhead",    "y",   "<10ms>",	   NULL	},
	{ CMD_SCC_TXTAIL,	"txtail",    "y",   "<10ms>", 	   NULL	},
	{ CMD_SCC_TXINVERT,  	"txinvert",  "b",   "<on|off>",	   NULL	},

	CMDTAB_END
};
 
static struct Command code cmdtab_master[] = {
	{ CMD_MASTER_ADD,	"add",	  "c|cc", "<path>",	     NULL },
	{ CMD_MASTER_CMODE,	"cmode",  "cb",	  "<call> <on|off>", NULL },
	{ CMD_MASTER_DEL,	"delete", "c",	  "<call>",	     NULL },
	{ CMD_MASTER_LIST,	"list",	  NULL,	  NULL,		     NULL },
	{ CMD_MASTER_PASSWD,	"passwd", "cT",	  "<call> [pwd]",    NULL },

	CMDTAB_END
};

static struct Command code cmdtab_sched[] = {
	{ CMD_SCHED_LOCK, 	"lock",   "t",	  "<slots...>", NULL	},
	{ CMD_SCHED_STATE, 	"state",  NULL,	  NULL,		NULL	},
	{ CMD_SCHED_TIME,	"time",	  NULL,   NULL,		NULL	},
	{ CMD_SCHED_QUEUE,	"queue",  NULL,	  NULL,		NULL	},
	{ CMD_SCHED_UNLOCK,	"unlock", "t",	  "<slots...>",	NULL	},

	CMDTAB_END
};

static struct Command code cmdtab_trigger[] = {
	{ CMD_TRIGGER_ADD,	  "add",	"l",	"<addr>",     NULL },
	{ CMD_TRIGGER_DEL,	  "delete",	"l",	"<addr>",     NULL },
	{ CMD_TRIGGER_LIST,	  "list",	NULL,   NULL,	      NULL },

	CMDTAB_END
};

static struct Command code cmdtab[] = {
	{ CMD_HELP,	"?",	     "C",    "[command]", NULL		},
	{ CMD_AX25,	"ax25",	     NULL,   NULL, 	  cmdtab_ax25 	},
	{ 
	  CMD_BOOTDEV,	"bootdev",   "s",    "<eprom|flash0|flash1>",	  
	  NULL		
	},
	{ CMD_CTEXT,	"ctext",     NULL,   NULL,	  NULL		},
	{ 
	  CMD_FLASH,	"flash",     "s",    "<flash0|flash1>",	  
	  NULL		
	},
	{ CMD_HELP,	"help",	     "C",    "[command]", NULL		},
	{ 
	  CMD_IDENTITY, "identity",  "lT",   "<0|addr> [[#|$|@]text]", 
	  NULL	
	},
	{ CMD_MASTER,	"master",    NULL,   NULL,	  cmdtab_master	},
	{ 
	  CMD_PAGE,	"page",	     "ulyT", "<bps> <addr> <func> <[#|$|@]text>",
	  NULL		
	},
	{ CMD_PARMS,    "parms",     NULL,   NULL,      NULL		},
	{ CMD_PASSWD,	"passwd",    "T",    "[pwd]",   NULL		},
	{ CMD_QUIT,	"quit",	     NULL,   NULL,	NULL		},
	{ CMD_RESET,	"reset",     NULL,   NULL,	NULL		},
	{ CMD_SCC,	"scc",	     NULL,   NULL,	cmdtab_scc	},
	{ CMD_SCHED,	"scheduler", NULL,   NULL,      cmdtab_sched	},
	{ CMD_STATE,	"state",     NULL,   NULL,	NULL		},
	{ CMD_SYS,	"sysop",     NULL,   NULL,	NULL		},
	{ CMD_TRACE,	"trace",     NULL,   NULL,      NULL		},
	{ CMD_TRIGGER,  "trigger",   NULL,   NULL,      cmdtab_trigger  },
	{ CMD_USAGE,	"usage",     NULL,   NULL,      NULL		},
	{ CMD_WRITE,	"write",     NULL,   NULL,	NULL		},
	
	CMDTAB_END
};


/*
 *	Einfach verkettete Liste aus allen Infoboxen
 *	(Infoboxtabelle) 
 *
 */
static struct Infobox xdata * iboxtab = NULL;

/*
 *	Im naechsten Zyklus zu bearbeitende Infobox.
 *
 */
static struct Infobox xdata * ibox_next = NULL;

/*---------------------------------------------------------------------------*/


/*
 *	Erzeugt eine neue Infobox, verbindet sie mit
 *	dem AX.25-Controlblock <axcb> und traegt sie in
 *	die Infobox-Tabelle ein. Wenn keine Infobox
 *	erzeugt werden kann, wird die AX.25-Verbindung
 *	wieder geschlossen.
 *
 */
void ibox_setup (struct AX25_cb xdata * axcb)
{
	struct Infobox xdata * idata ibox;

	ibox = (struct Infobox xdata *)malloc(sizeof(struct Infobox));

	if (!ibox) {
		/* 
		 *	Wenn kein Fehler auf dem Link zum Peer existiert,
		 *	wird die Fehlermeldung evtl. korrekt ausgesendet.
		 *	Falls das System total ueberlastet ist, wird
		 *	die Fehlermeldung voraussichtlich garnicht
		 *	gesendet -> out of memory.
		 *
		 */
		ax25_printf(axcb,"Sorry, system overloaded!\r");
		ax25_close(axcb);
		return;
	}

	/*
	 *	Meldung an den User schicken
	 *
	 */
	ax25_printf(axcb, VERSION_STRING "%c%s" IBOX_PROMPT,
			    (char)(*conf.ctext ? ' ' : '\r'),
	                    (__XPTR__)conf.ctext);

	ibox->axcb  = axcb;
	ibox->sysop = 0;
	ibox->mode  = IBOX_MODE_CMD;
	
	ibox->next = iboxtab;
	iboxtab = ibox;
}



/*
 *	Schliesst die Infobox-Verbindung, traegt die Infobox
 *	aus der Infobox-Tabelle aus und gibt alle von der Infobox
 *	reservierten Resourcen wieder frei.
 *
 */
static void ibox_shutdown (struct Infobox xdata * ibox)
{
	struct Infobox xdata * idata ib;
	struct Infobox xdata * idata prev;

	if (ibox == ibox_next)
		ibox_next = ibox_next->next;

	prev = NULL;

	for (ib = iboxtab; ib != NULL && ib != ibox; ib = ib->next) 
		prev = ib;

	if (!prev)
		iboxtab = iboxtab->next;
	else
		prev->next = ibox->next;
		
	ax25_close(ibox->axcb);

	switch (ibox->mode) {
	case IBOX_MODE_PASSWD:
		pw_delete(ibox->obj);
		break;

	case IBOX_MODE_INPUT_CTEXT:
		inp_delete(ibox->obj);
		break;

	case IBOX_MODE_FLASH:
		up_delete(ibox->obj);
		break;
	
	case IBOX_MODE_TRACE:
		trace_delete();
	}

	free(ibox);
}


/*---------------------------------------------------------------------------*/


/*
 * 	Wandelt den String <str> in eine numerische
 *	Bank-Angabe um. Wenn die Umwandlung nicht gelingt,
 *	wird -1 zurueckgegeben.
 *
 */
static char ibox_strtobank (const char xdata *str)
{
	if (strcmp_xc(str, "flash0") == 0)
		return BOOT_FLASH0;

	if (strcmp_xc(str, "flash1") == 0)
		return BOOT_FLASH1;

	if (strcmp_xc(str, "eprom") == 0)
		return BOOT_EPROM;

	return -1;
}


/*
 *	Wandelt die Datenrate <bps> in eine
 *	von dem SCC-Modul verstandene speed-Angabe
 *	(SCC_B512, SCC_B1200, etc.) um.
 *	Im Fehlerfall wird -1 zurueckgegeben.
 *
 */
static char ibox_bps_to_speed (unsigned int bps)
{
	switch (bps) {
	case 512:
		return SCC_B512;

	case 1200:
		return SCC_B1200;

	case 2400:
		return SCC_B2400;
	}

	return -1;
}


/*---------------------------------------------------------------------------*/


/*
 *	PAGE
 *
 *	Funktion fuehrt alle Aktionen aus, die fuer das Kommando
 *	"page" ausgefuehrt werden muessen.
 *
 */
static void ibox_cmd_page (struct Infobox xdata *ibox,
			   struct Args xdata *args)
{
	struct Page xdata *p;
	char speed;
	
	if (!args->tail) {
		ax25_printf(ibox->axcb, "page text missing\r");
		return;
	}

	speed = ibox_bps_to_speed(args->argv[0].u16);

	if (speed == -1) {
		ax25_printf(ibox->axcb, "bit rate not supported\r");
		return;
	}

	p = poc_encode(args->argv[1].u32,	/* addr */
		       args->argv[2].u8,	/* func */
		       args->tail);		/* text */

	if (!p) {
		ax25_printf(ibox->axcb, "failed to encode page\r");
		return;
	}

	if (!sched_addpage(speed, p, 1)) {
		ax25_printf(ibox->axcb, "failed to queue page\r");
		page_delete(p);
		return;
	}

	ax25_printf(ibox->axcb, "page queued\r");
}


/*
 *	Gibt die Rufzeichen aller Infobox-Nutzer aus
 *
 */
static void ibox_usage (struct AX25_cb xdata *axcb)
{
	struct Infobox xdata *ibox;

	ax25_printf(axcb, "infobox:\r");

	for (ibox = iboxtab; ibox != NULL; ibox = ibox->next)
		ax25_printf(axcb, 
		 	    "%a\r", 
			    (__XPTR__)&ibox->axcb->path.path.addr.peer);
		
}


/*
 *	Dispatcher fuer alle ROOT-Kommandos
 *
 */
static char ibox_dispatch_root (struct Infobox xdata *ibox, 
                                struct Args xdata *args)
{
	char bank;

	switch (args->id) {
	case CMD_BOOTDEV:
		if ( (bank = ibox_strtobank(args->argv[0].string)) == -1 ) {
			ax25_printf(ibox->axcb, "invalid boot device\r");
			break;
		}

		if (!boot_set_bank(bank))
			ax25_printf(ibox->axcb, 
			            "failed to activate boot device\r");
		else
			ax25_printf(ibox->axcb, "boot device activated\r");
		break;
	
	case CMD_CTEXT:
		if ( (ibox->obj = inp_new(ibox->axcb, conf.ctext, 
					  sizeof(conf.ctext))) != NULL )
			ibox->mode = IBOX_MODE_INPUT_CTEXT;
		else
			ax25_printf(ibox->axcb, 
				    "please retry later\r");
		break;

	case CMD_FLASH:
		if ( (bank = ibox_strtobank(args->argv[0].string)) == -1 ||
		     (bank != BOOT_FLASH0 && bank != BOOT_FLASH1) ) 
		{
			ax25_printf(ibox->axcb, "invalid flash device\r");
			break;
		}

		if (IS_CSEG_FLASH()) {
			ax25_printf(ibox->axcb, "sri, flash is active\r");
			break;
		}
	
		if ( (ibox->obj = up_new(ibox->axcb, bank)) )
			ibox->mode = IBOX_MODE_FLASH;
		else	
			ax25_printf(ibox->axcb,
			            "please retry later\r");
		break;
	
	case CMD_HELP:
		if (!args->cmd) {
			ax25_printf(ibox->axcb, "Commands:\r");
			cmd_showcmds(ibox->axcb, cmdtab);
			break;
		}

		if (args->cmd->subtab) {
			ax25_printf(ibox->axcb, "Subcommands:\r");
			cmd_showcmds(ibox->axcb, args->cmd->subtab);
			break;
		}

		cmd_show_usage(ibox->axcb, args->cmd);
		break;

	case CMD_IDENTITY:
		if (!args->tail) {
			*conf.ident_text = 0;
		} else if (strlen_x(args->tail) >= sizeof(conf.ident_text)) {
			ax25_printf(ibox->axcb, 
			            "ident text too long (max. %u chars)\r",
				    (unsigned int)sizeof(conf.ident_text) - 1);
			break;
		} else {
			strcpy_xx(conf.ident_text, args->tail);	
		}
	
		conf.ident_addr = args->argv[0].u32;
		break;

	case CMD_PAGE:
		ibox_cmd_page(ibox,args);
		break;

	case CMD_PARMS:
		if (conf.ident_addr)
			ax25_printf(ibox->axcb,
				    "addr     text\r"
				    "%-9lu%s\r",
				    (unsigned long)conf.ident_addr,
				    (__XPTR__)conf.ident_text);
		else
			ax25_printf(ibox->axcb,
				    "identity not set\r");
		break;

	case CMD_PASSWD:
		if (!args->tail) {
			*conf.hot_data.passwd = 0;
			ax25_printf(ibox->axcb, "password deleted\r");
		} else {
			unsigned int len;

			len = strlen_x(args->tail);

			if (len < PW_LEN_MIN) {
				ax25_printf(ibox->axcb, "password too short\r");
				break;
			}

			if (len >= sizeof(conf.hot_data.passwd)) 
			{
				ax25_printf(ibox->axcb, "password too long\r");
				break;
			}
				
			strcpy_xx(conf.hot_data.passwd, args->tail);
		}
		
		break;

	case CMD_QUIT:
		ax25_printf(ibox->axcb, "73!\r");
		return 1;
		
	case CMD_RESET:
		RESET();
		break;

	case CMD_STATE:
		ax25_printf(ibox->axcb,
			    "free memory:   %u bytes\r" 
			    "malloc faults: %u\r"
			    "stack hwm:     %u\r",
			    (unsigned int)mm_coreleft,
			    (unsigned int)mm_mallocfails,
			    (unsigned int)sys_hwm());

		if (flash_type != FLASH_UNTESTED)
			ax25_printf(ibox->axcb,
		    		    "flash man/dc:  0x%02X 0x%02X\r",
				    (unsigned int)flash_manufacturer,
			    	    (unsigned int)flash_device_code);

		ax25_printf(ibox->axcb,
			    "\r"
			    "ax25 sockets:  %u\r"
			    "\r"
			    "ptt dcd con act\r"
			    "%3b%4b%4b%4b\r\r",
			    (unsigned int)ax25_cbcnt,
			    (unsigned char)IS_ON(LED_PTT),
			    (unsigned char)IS_ON(LED_DCD),
			    (unsigned char)IS_ON(LED_CON),
			    (unsigned char)IS_ON(LED_ACT));

		boot_print_state(ibox->axcb);
		break;

	case CMD_SYS:
		if (!*conf.hot_data.passwd) {
			ibox->sysop = 1;
			break;
		}

		ibox->obj = pw_new(ibox->axcb, conf.hot_data.passwd, 1);
		
		if (ibox->obj != NULL)
			ibox->mode = IBOX_MODE_PASSWD;

		break;

	case CMD_TRACE:
		if (!trace_new(ibox->axcb)) {
			ax25_printf(ibox->axcb, "failed to activate trace\r");
			break;
		}

		ax25_printf(ibox->axcb, "trace activated, press <enter> to terminate\r");
		ibox->mode = IBOX_MODE_TRACE;
		break;

	case CMD_USAGE:
		ibox_usage(ibox->axcb);
		ax25_printf(ibox->axcb, "\r");
		scp_usage(ibox->axcb);
		break;

	case CMD_WRITE:
		conf_save();
		ax25_printf(ibox->axcb,"settings written\r");
		break;
		
	default:
		return -1;
	}

	return 0;
}


/*
 *	Dispatcher fuer alle AX25-Kommandos
 *
 */
static char ibox_dispatch_ax25 (struct Infobox xdata *ibox, 
                                struct Args xdata *args)
{
	switch (args->id) {
	case CMD_AX25_MYCALL:
		memcpy_xx(&conf.hot_data.mycall, &args->argv[0].call,
			  sizeof(struct AX25_address));
		break;

	case CMD_AX25_N2:
		conf.l2_n2 = args->argv[0].u8;
		break;

	case CMD_AX25_PARMS:
		ax25_printf(ibox->axcb,
		            "mycall      n2    t1    t2    t3  unack\r"
			    "%-9a %4u %4us %2u.%us %4us %6u\r", 
			    (__XPTR__)&conf.hot_data.mycall,
			    (unsigned int)conf.l2_n2,
			    (unsigned int)conf.l2_t1,
			    (unsigned int)conf.l2_t2 / 10,
			    (unsigned int)conf.l2_t2 % 10,
			    (unsigned int)conf.l2_t3 * 10,
			    (unsigned int)conf.l2_unack);
		break;

	case CMD_AX25_T1:
		conf.l2_t1 = args->argv[0].u8;
		break;
		
	case CMD_AX25_T2:
		conf.l2_t2 = args->argv[0].u8;
		break;
		
	case CMD_AX25_T3:
		conf.l2_t3 = args->argv[0].u8;
		break;
	
	case CMD_AX25_UNACK:
		if (args->argv[0].u8 < 1 || args->argv[0].u8 > 7) {
			ax25_printf(ibox->axcb, 
				    "value out of range\r");
			break;
		}

		conf.l2_unack = args->argv[0].u8;
		break;

	default:
		return -1;
	}

	return 0;
}


/*
 *	Dispatcher fuer alle SCC-Kommandos
 *
 */
static char ibox_dispatch_scc (struct Infobox xdata *ibox, 
                               struct Args xdata *args)
{
	switch (args->id) {
	case CMD_SCC_CALIBRATE:
		if (args->argv[0].u16 > 2400) {
			ax25_printf(ibox->axcb,
			            "value out of range\r");
			break;
		}
	
		if (!scc_calibrate(args->argv[0].u16,
		                   (args->argc == 2) ? args->argv[1].u8 : 0))
		{				   
			ax25_printf(ibox->axcb, "sri, scc-tx busy\r");
		}			
		break;
	 
	case CMD_SCC_PARMS:
		ax25_printf(ibox->axcb,
		            "txhead  txtail  txinvert  preamble  rxinvert\r"
			    "%4ums %5ums %9b %9u %9b\r",
			    (unsigned int)conf.scc_txhead * 10,
			    (unsigned int)conf.scc_txtail * 10,
			    (unsigned char)conf.scc_txinvert,
			    (unsigned int)conf.scc_preamble,
			    (unsigned char)conf.scc_rxinvert);
		break;

	case CMD_SCC_PREAMBLE:
		conf.scc_preamble = args->argv[0].u16;
		break;

	case CMD_SCC_RXINVERT:
		conf.scc_rxinvert = args->argv[0].boolean;
		break;
		
	case CMD_SCC_TXHEAD:
		conf.scc_txhead = args->argv[0].u8;
		break;
		
	case CMD_SCC_TXTAIL:
		conf.scc_txtail = args->argv[0].u8;
		break;
	
	case CMD_SCC_TXINVERT:
		conf.scc_txinvert = args->argv[0].boolean;
		break;

	default:
		return -1;
	}

	return 0;
}


/*
 *	Dispatcher fuer alle MASTER-Kommandos
 *
 */
static char ibox_dispatch_master (struct Infobox xdata *ibox, 
                                  struct Args xdata *args)
{
	struct AX25_smart_path xdata path;
	struct Master xdata *m;

	switch (args->id) {
	case CMD_MASTER_ADD:
		memcpy_xx(&path.peer,    &args->argv[0].call, 
		          sizeof(path.peer));
			  
		memcpy_xx(&path.digi[0], &args->argv[1].call,
			  sizeof(path.digi[0]));
			  
		memcpy_xx(&path.digi[1], &args->argv[2].call,
			  sizeof(path.digi[1]));

		path.digi_cnt = args->argc - 1;
	
		if (!m_add(&path))
			ax25_printf(ibox->axcb, "failed to add\r");

		break;

	case CMD_MASTER_CMODE:
		if ((m = m_lookup(&args->argv[0].call)) == NULL) {
			ax25_printf(ibox->axcb, "unknown master\r");
			break;
		}

		m->cmode = (args->argv[1].boolean);
		break;
	
	case CMD_MASTER_DEL:
		if (!m_del(&args->argv[0].call))
			ax25_printf(ibox->axcb, "unknown master\r");

		break;
	
	case CMD_MASTER_LIST:
		m_list(ibox->axcb);
		break;

	case CMD_MASTER_PASSWD:
		if ((m = m_lookup(&args->argv[0].call)) == NULL) {
			ax25_printf(ibox->axcb, "unknown master\r");
			break;
		}

		if (!args->tail) {
			*m->passwd = 0;
			ax25_printf(ibox->axcb, "password deleted\r");
		} else {
			unsigned int len;

			len = strlen_x(args->tail);

			if (len < PW_LEN_MIN) {
				ax25_printf(ibox->axcb, "password too short\r");
				break;
			}

			if (len >= sizeof(m->passwd)) 
			{
				ax25_printf(ibox->axcb, "password too long\r");
				break;
			}
				
			strcpy_xx(m->passwd, args->tail);
		}

		break;
	
	default:
		return -1;
	}

	return 0;
}


/*
 *	Dispatcher fuer alle SCHEDULER-Kommandos
 *
 */
static char ibox_dispatch_sched (struct Infobox xdata *ibox, 
                                 struct Args xdata *args)
{
	stime_t time;

	switch (args->id) {
	case CMD_SCHED_LOCK:
		sched_lock(args->argv[0].u16);
		break;

	case CMD_SCHED_STATE:
		sched_print_state(ibox->axcb);
		break;
		
	case CMD_SCHED_TIME:
		time = rtc_time();

		ax25_printf(ibox->axcb,
			    "stime  cyclus  slot  elapsed\r"
			    "%5u %7u %5u %6u.%u\r",
			    (unsigned int)time,
			    (unsigned int)SCHED_CYCL(time),
			    (unsigned int)SCHED_SLOT(time),
			    (unsigned int)(SCHED_ELAP(time) / 10),
			    (unsigned int)(SCHED_ELAP(time) % 10));
		break;

	case CMD_SCHED_QUEUE:
		sched_print_qstate(ibox->axcb);
		break;
		
	case CMD_SCHED_UNLOCK:
		sched_unlock(args->argv[0].u16);
		break;
	
	default:
		return -1;
	}

	return 0;
}


/*
 *	Dispatcher fuer alle Trigger-Kommandos
 *
 */
static char ibox_dispatch_trigger (struct Infobox xdata *ibox, 
                                   struct Args xdata *args)
{
	switch (args->id) {
	case CMD_TRIGGER_ADD:
		if (!rp_trigger_add(args->argv[0].u32))
			ax25_printf(ibox->axcb, "failed to add trigger\r");
		break;

	case CMD_TRIGGER_DEL:
		if (!rp_trigger_del(args->argv[0].u32))
			ax25_printf(ibox->axcb, "failed to delete trigger\r");
		break;

	case CMD_TRIGGER_LIST:
		rp_trigger_list(ibox->axcb);
		break;

	default:
		return -1;
	}

	return 0;
}



/*
 *	Wertet die Rueckgabe des Kommandointerpreters
 *	aus.
 *
 *	Diese Funktion gibt 1 zurueck, wenn die Infobox
 *	beendet werden soll. Ansonsten wird 0 zurueckgegeben.
 *
 */
static char ibox_dispatch (struct Infobox xdata *ibox, 
                           struct Args xdata *args)
{
	char rc;

	if ( (args->id & CMD_SYSOP) && !ibox->sysop ) {
		ax25_printf(ibox->axcb, "permission denied\r");
		return 0;
	}

	switch (args->id & CMD_GROUP_MASK) {
	case CMD_ROOT:
		rc = ibox_dispatch_root(ibox, args);
		break;
		
	case CMD_AX25:
		rc = ibox_dispatch_ax25(ibox, args);
		break;
		
	case CMD_SCC:
		rc = ibox_dispatch_scc(ibox, args);
		break;
		
	case CMD_MASTER:
		rc = ibox_dispatch_master(ibox, args);
		break;

	case CMD_SCHED:
		rc = ibox_dispatch_sched(ibox, args);
		break;

	case CMD_TRIGGER:
		rc = ibox_dispatch_trigger(ibox, args);
		break;
		
	default:
		rc = -1;
	}

	if (rc == -1) {
		ax25_printf(ibox->axcb,"command not implemented\r");
		return 0;
	}

	return rc;
}


/*
 *	Eingabe des User auswerten und die Kommandos
 *	ausfuehren.
 *
 */
static void ibox_cmd_mode (struct Infobox xdata *ibox)
{
	struct Args xdata *args;
	char xdata * str;

	if ( !(str = ax25_gets(ibox->axcb, 80)) )
		return;

	ax25_printf(ibox->axcb, "\r");
       
	args = cmd_interprete(ibox->axcb, str, cmdtab);

	if (args != NULL)
		if (ibox_dispatch(ibox, args)) {
			free(str);
			ibox_shutdown(ibox);
			return;
		}
	
	free(str);

	if (ibox->mode == IBOX_MODE_CMD)
		ax25_printf(ibox->axcb, IBOX_PROMPT);
}


/*
 *	Bearbeitet die naechste anstehende Infobox 
 *
 */
void ibox_run ()
{
	struct Infobox xdata * idata ibox;
	char rc;

	if (!ibox_next) {
		if (!iboxtab)
			return;
	
		ibox_next = iboxtab;
	}

	ibox = ibox_next;
	ibox_next = ibox_next->next;

	switch (ax25_event(ibox->axcb)) {
	case EVENT_LINKRESET:
		ibox->sysop = 0;
		return;

	case EVENT_LINKFAILURE:
	case EVENT_DISCONNECT:
		ibox_shutdown(ibox);
		return;

	case EVENT_TXQEMPTY:
		/*
		 *	Nur wenn die TX-Queue des AX.25-CBs leer
		 *	ist, duerfen Kommandos abgearbeitet werden.
		 *
		 */
		break;

	default:
		return;
	}

	/*
	 *	Je nach mode muessen unterschiedliche
	 *	Aktionen ausgefuehrt werden.
	 *
	 */
	switch (ibox->mode) {
	case IBOX_MODE_CMD:
		ibox_cmd_mode(ibox);
		break;

	case IBOX_MODE_PASSWD:
		if ( (rc = pw_run(ibox->obj)) != 0 ) {
			pw_delete(ibox->obj);
			ibox->sysop = (rc == 1);
			ibox->mode = IBOX_MODE_CMD;
			ax25_printf(ibox->axcb, IBOX_PROMPT);
		}
		break;

	case IBOX_MODE_INPUT_CTEXT:
		if (inp_run(ibox->obj)) {
			inp_delete(ibox->obj);
			ibox->mode = IBOX_MODE_CMD;
			ax25_printf(ibox->axcb, IBOX_PROMPT);
		}
		break;

	case IBOX_MODE_FLASH:
		if (up_run(ibox->obj)) {
			up_delete(ibox->obj);
			ibox->mode = IBOX_MODE_CMD;
			ax25_printf(ibox->axcb, IBOX_PROMPT);
		}
		break;

	case IBOX_MODE_TRACE:
		if (trace_run()) {
			trace_delete();
			ibox->mode = IBOX_MODE_CMD;
			ax25_printf(ibox->axcb, IBOX_PROMPT);
		}
		break;

	default:
		ibox_shutdown(ibox);
		return;
	}
}



