
/*
 *	$Id: input.c,v 1.1.1.1 2002/02/15 22:36:40 Klaus Exp $
 *
 *	input.c		Modul zur mehrzeiligen Texteingabe
 *
 *
 *
 *
 *	(c) 2001,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "input.h"
#include "mm.h"
#include "ax25.h"
#include "ax25_io.h"
#include "string.h"

/*
 *	Zustaende des input-Objekts
 *
 */
#define STATE_PRINT_USAGE	0
#define STATE_READ		1
#define STATE_COPY		2
#define STATE_READY		3

/*--------------------------------------------------------------------------*/


/*
 *	Erzeugt ein neues Eingabeobjekt und gibt den Zeiger
 *	darauf zurueck. Wenn das Objekt nicht erzeugt werden konnte,
 *	wird NULL zurueckgegeben.
 *
 */
struct Input xdata * inp_new (struct AX25_cb xdata *axcb,
			      char xdata *buf,
			      unsigned int buflen)
{
	struct Input xdata *inp;

	if ( !(inp = malloc(sizeof(struct Input) + buflen)) )
		return NULL;

	inp->state = STATE_PRINT_USAGE;
	inp->axcb  = axcb;
	inp->buf   = buf;
	inp->wptr  = (char xdata *)(inp + 1);

	/*
	 *	Die abschliessende Null wird hier direkt abgezogen.
	 *	Das vereinfacht die Berechnungen bei der Texteingabe
	 *	sehr stark.
	 *
	 */
	inp->left = buflen - 1;

	return inp;
}


/*
 *	Loescht das Input-Objekt <inp>
 *
 */
void inp_delete (struct Input xdata *inp)
{
	free(inp);
}


/*
 *	Liest den Text zeilenweise ein und speichert ihn
 *	im temporaeren Puffer.
 *
 */
static void inp_read (struct Input xdata *inp)
{
	char xdata *str;
	unsigned int len;

	if ( !(str = ax25_gets(inp->axcb, 160)) )
		return;

	if (!strcmp_xc(str, "/EX")) {
		free(str);
		inp->state = STATE_COPY;
		return;
	}

	if (!inp->left) {
		free(str);
		return;
	}

	len = strlen_x(str);

	if (len > inp->left) {
		str[inp->left] = 0;
		len = inp->left;
	}

	strcpy_xx(inp->wptr, str);
	inp->wptr += len;
	inp->left -= len;

	if (inp->left) {
		*inp->wptr++ = '\r';
		inp->left--;
	}
	
	free(str);
}


/*
 *	Liest zeilenweise die Eingabe des Users ein und
 *	speichert sie im temporaeren Puffer.
 *
 *	Rueckgabe:
 *	0	Eingabe noch nicht abgeschlossen
 *	1	Eingabe fertig
 *
 */
char inp_run (struct Input xdata *inp)
{
	switch (inp->state) {
	case STATE_PRINT_USAGE:
		ax25_printf(inp->axcb, 
			    "%u bytes free\r"
			    "Please enter text. Terminate with /EX\r",
			    (unsigned int)inp->left);
		inp->state = STATE_READ;
		break;

	case STATE_READ:
		inp_read(inp);
		break;

	case STATE_COPY:
		*inp->wptr = 0;
		strcpy_xx(inp->buf, (char xdata *)(inp + 1));
		inp->state = STATE_READY;
		break;

	case STATE_READY:
	default:
		return 1;
	}

	return 0;
}



