
/*
 *	$Id: listener.c,v 1.1.1.1 2002/02/15 22:36:40 Klaus Exp $
 *
 *	listener.c	Modul nimmt AX.25-Verbindungen entgegen und
 *			leitet sie an den zugehoerigen Port weiter.
 *
 *
 *
 *
 *	(c) 2001,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "listener.h"
#include "ax25.h"
#include "mm.h"
#include "infobox.h"
#include "master.h"
#include "scp.h"

/*
 *	Dieser AX.25-Block wartet immer auf Verbindungswuensche von
 *	aussen.
 *
 */
static struct AX25_cb xdata * listen_axcb = (struct AX25_cb xdata *)NULL;

/*---------------------------------------------------------------------------*/


/*
 *	Erzeugt ein listen-Controlblock, wenn keiner existiert. Wenn
 *	ein Verbindungswunsch angekommen ist, wird eine Verbindung
 *	hergestellt und der AX.25-Controlblock mit dem zugehoerigen
 *	Port verbunden.
 * 
 */
void l_run ()
{
	if (!listen_axcb) {
		if ( (listen_axcb = ax25_open(TYPE_SEQPACKET)) )
			ax25_listen(listen_axcb);

		return;
	}

	switch (ax25_event(listen_axcb)) {
	case EVENT_NOEVENT:
	case EVENT_TXQEMPTY:
		break;

	case EVENT_CONREQ:
		if (!ax25_accept(listen_axcb))
			break;

		/*
		 *	Wenn fuer das Quell-Call ein Eintrag in der
		 *	Master-Tabelle existiert, wird zum SCP
		 *	verzweigt, ansonsten zur Infobox.
		 *
		 */
		if (m_lookup(&listen_axcb->path.path.addr.peer))
			scp_setup(listen_axcb);
		else
			ibox_setup(listen_axcb);
	
		/*
		 *	Listen-CB zuruecksetzten, damit auf den
		 *	naechsten Connect gehoert wird.
		 *
		 */
		listen_axcb = (struct AX25_cb xdata *)NULL;
		break;

	default:
		/*
		 *	Alle anderen Events duerfen bei korrektem
		 *	Betrieb nicht vorkommen. Wenn doch, wird
		 *	hier versucht die Firmware wieder zu
		 *	stabilisieren.
		 *
		 */
		ax25_close(listen_axcb);
		listen_axcb = (struct AX25_cb xdata *)NULL;
	}
}




