
/*
 *	$Id: mm.c,v 1.5 2002/02/21 21:36:24 klaus Exp $
 *
 *	mm.c		Memory Manager	
 *
 *			Implementation einer ganz einfachen
 *			Speicherverwaltung.
 *
 *
 *
 *	(c) 1999,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "mm.h"
#include "system.h"


struct BlockHeader {
	struct BlockHeader	xdata *prev;
	struct BlockHeader	xdata *next;
	struct BlockHeader	xdata *gap;

	/*
	 *	Hier folgt der Speicher, der durch den Block-Header
	 *	verwaltet wird.
	 *
	 */
};


/*
 *	Kompletter Speicher, der verwaltet wird.
 *
 */
#ifdef SDCC
struct _MEM_ {
	struct BlockHeader 	xdata firstblk;
	unsigned char  		xdata mem_pool[MEMSIZE];
	struct BlockHeader  	xdata lastblk;
};
#else
struct _MEM_ {
	struct BlockHeader 	firstblk;
	unsigned char  		mem_pool[MEMSIZE];
	struct BlockHeader  	lastblk;
};
#endif

static struct _MEM_ xdata mem;


/*
 *	Freier Speicher in Bytes.
 *
 */
unsigned int mm_coreleft;

/* 
 *	Anzahl der malloc-Fehler
 *
 */
unsigned int mm_mallocfails;


/*---------------------------------------------------------------------------*/



/*
 *	Initialisiert die Block-Tabelle
 *
 */
void mm_init ()
{
	mem.firstblk.prev = NULL;
	mem.firstblk.next = &mem.lastblk;
	mem.firstblk.gap  = &mem.firstblk + 1;

	mem.lastblk.prev  = &mem.firstblk;
	mem.lastblk.next  = NULL;
	mem.lastblk.gap   = NULL;

	mm_coreleft       = MEMSIZE;
	mm_mallocfails    = 0;
}



/*
 *	Reserviert size-Bytes Speicher. Wenn ein sogrosser Speicherblock
 *	nicht frei ist, wird NULL zurueckgegeben.
 *
 */
void xdata * mm_malloc (unsigned int size)
{
	struct BlockHeader xdata * data nblk;
	struct BlockHeader xdata * data blk;
	unsigned int data gap_size;

	size += sizeof(struct BlockHeader);

	for (blk = &mem.firstblk; blk != &mem.lastblk; blk = blk->next) {

		/*
		 *	Zeiger auf neuen moeglichen Block ermitteln
		 *
		 */
		nblk = blk->gap;

		gap_size = (char xdata *)blk->next - (char xdata *)nblk;

		if (gap_size >= size) {
			nblk->prev = blk;
			nblk->next = blk->next;
			
			blk->next = nblk;
			nblk->next->prev = nblk;

			nblk->gap = (struct BlockHeader xdata *)
			            ((char xdata *)nblk + size);
	
			mm_coreleft -= size;
	
			return (void xdata *)(nblk + 1);
		}
	}

	/*
	 *	Wenn OOM_THRESH-Mal erfolglos versucht wurde Speicher
	 *	zu reservieren wird die Firmware vorsorglich neu
	 *	gestartet.
	 *
	 */
	if (++mm_mallocfails >= OOM_THRESH)
		RESET();

	return NULL;
}



/*
 *	Gibt den Speicher *mem wieder frei
 *
 */
void mm_free (void xdata *mem)
{
	struct BlockHeader xdata * idata blk;

	blk = ((struct BlockHeader xdata *)mem) - 1;

	mm_coreleft += (char xdata *)blk->gap - (char xdata *)blk;
	
	blk->prev->next = blk->next;
	blk->next->prev = blk->prev;
}



