
#ifndef _PAGE_H_
#define _PAGE_H_

/*
 *	Maximale Anzahl von Codewoertern pro Funkruf
 *
 */
#define PAGE_CWS_MAX		255

/*
 * 	Die unterschiedlichen Funkruf-Domaenen
 *
 */
#define PAGE_DOMAIN_PAGER	0	/* Funkruf an Pager */
#define PAGE_DOMAIN_SYSTEM	1	/* Funkruf von System zu System */
#define PAGE_DOMAIN_BROADCAST	2	/* Broadcast-Funkruf */


struct PageSequence {
	/*
	 *	Anzahl der Codewoetert im aktuellen Batch, die vor
	 *	dem ersten in dieser PageSequence gespeicherten
	 *	Codewoerter voranstehen. Dieser Offset wird benoetigt,
	 *	um die Frame-Nummer korrekt zu berechnen.
	 *
	 */
	unsigned char		cw_off;

	/*
	 *	Puffer mit Codewoetern <cw_buf> und Anzahl der
	 *	Codewoerter <cw_cnt> im Puffer.
	 *
	 */
	unsigned long xdata	*cw_buf;
	unsigned int		cw_cnt;
};


struct Page {
	struct Page xdata	*next;

	/*
	 *	Frame, in dem der Funkruf anfaengt.
	 *
	 */
	unsigned char		frame : 4;

	/*
	 *	Domaenenzugehoerigkeit des Funkrufs. Diese
	 *	Variable gibt an, an welche Zielgruppe
	 *	der Funkruf gerichtet ist.
	 *
	 */
	unsigned char		domain : 4;

	/*
	 *	cw_cnt = Anzahl der Codewoerter im cw_buf
	 *	cw_buf = Zeiger auf Codewoerter
	 *
	 */
	unsigned char		cw_cnt;
	unsigned long xdata	*cw_buf;

	/*
	 *	Hier folgt der Codewort-Puffer
	 *
	 */
};


struct PageQueue {
	struct Page xdata	*first;
	struct Page xdata	*last;

	unsigned int		page_cnt;
};


struct Page xdata * page_new (unsigned char cw_cnt, unsigned char frame);
struct Page xdata * page_split (struct PageSequence xdata *ps,
			        unsigned char cnt);

void pq_init (struct PageQueue xdata *pq);
void pq_clear (struct PageQueue xdata *pq);
void pq_clear_dom (struct PageQueue xdata *pq, unsigned char domain);

void pq_enqueue (struct PageQueue xdata *pq, 
		 struct Page xdata *page, 
		 unsigned char head);
		  
struct Page xdata * pq_dequeue (struct PageQueue xdata *pq);

void pq_batch (struct PageQueue xdata *dest,
	       struct PageQueue xdata *src, 
	       unsigned char max_bcnt);
		       
void pq_insert (struct PageQueue xdata * dest,
		struct PageQueue xdata * src);


#define page_delete(p)		free(p)

#define pq_page_cnt(pq)		((pq)->page_cnt)
#define pq_first(pq)		((pq)->first)
#define pq_last(pq)		((pq)->last)
#define pq_next(p)		((p)->next)

#endif


