
/*
 *	$Id: passwd.c,v 1.1.1.1 2002/02/15 22:36:40 Klaus Exp $
 *
 *	passwd.c	BayCom Authentifizierungsverfahren
 *
 *
 *
 *
 *	(c) 2001,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "passwd.h"
#include "mm.h"
#include "ax25.h"
#include "ax25_io.h"
#include "system.h"
#include "config.h"
#include "string.h"

/*
 *	States der BayCom-Passwort State-Machine
 *
 */
#define STATE_GEN_NUMS		0	
#define STATE_PROMPT		1
#define STATE_CHECK		2
#define STATE_AUTH_SUCCESS	3
#define STATE_AUTH_FAILED	4

/*--------------------------------------------------------------------------*/


/*
 *	Neues Passwort-Authentifizierungs-Objekt erzeugen
 *	und einen Zeiger darauf zurueckgeben. Wenn kein Objekt
 *	erzeugt werden kann, wird NULL zurueckgegeben.
 *
 */
struct Passwd xdata * pw_new (struct AX25_cb xdata *axcb, 
			      char xdata *pw,
			      char prefix)
{
	struct Passwd xdata *p;
	unsigned int pwlen;

	pwlen = strlen_x(pw);

	if (pwlen < PW_LEN_MIN || pwlen > PW_LEN_MAX)
		return NULL;

	if ( !(p = (struct Passwd xdata *)malloc(sizeof(struct Passwd))) )
		return NULL;

	p->state  = STATE_GEN_NUMS;
	p->prefix = prefix;
	p->axcb   = axcb;
	p->buflen = 0;
	p->pw     = pw;
	p->pwlen  = pwlen; 

	return p;
}


/*
 *	Loescht das Passwort-Objekt <p>
 *
 */
void pw_delete (struct Passwd xdata *p)
{
	free(p);
}


/*
 *	Generiert nacheinander Zufallszahlen zur Indizierung
 *	des Passwort-Strings. Wichtig ist. dass die Zahlen
 *	im richtigen Interval liegen und dass sie nicht
 *	doppelt vorhanden sind.
 *
 */
static void pw_gen_nums (struct Passwd xdata *p)
{
	unsigned char rand;
	unsigned char n;

	/*
	 *	Das Low-Byte des Timers2 muss als
	 *	Zufallsgenerator herhalten.
	 *
	 */
	rand = TML2;
	rand %= p->pwlen;

	for (n = 0; n < p->buflen; n++)
		if (p->buf[n] == rand)
			return;

	p->buf[p->buflen++] = rand;

	if (p->buflen >= PW_NUM_CNT)
		p->state = STATE_PROMPT;
}


/*
 *	Gibt den Passwort-Prompt aus.
 *
 */
static void pw_prompt (struct Passwd xdata *p)
{
	unsigned char n;

	if (p->prefix)
		ax25_printf(p->axcb, "%m>");

	for (n = 0; n < PW_NUM_CNT; n++)
		ax25_printf(p->axcb, " %u", (unsigned int)p->buf[n] + 1);

	ax25_printf(p->axcb, "\r");
	
	p->state = STATE_CHECK;
}


/*
 *	Liesst die User-Eingabe ein und prueft, ob sich
 *	das zuvor generierte Passwort-Fragment darin
 *	befindet.
 *
 */
static void pw_check (struct Passwd xdata *p)
{
	char xdata *str;
	char xdata frag[PW_NUM_CNT + 1];
	unsigned char n;

	if ( !(str = ax25_gets(p->axcb, 80)) )
		return;

	for (n = 0; n < PW_NUM_CNT; n++)
		frag[n] = p->pw[p->buf[n]];

	frag[n] = 0;

	if (strstr_xx(str, frag))
		p->state = STATE_AUTH_SUCCESS;
	else
		p->state = STATE_AUTH_FAILED;

	free(str);
}


/*
 *	Arbeitet die Authentifizierungs-Statemachine ab.
 *
 *	Rueckgabe:
 *	0	Unter-Interaktion laeuft noch
 *	1	Authentifizierung erfolgreich
 *	-1	Authentifizierung fehlgeschlagen
 *
 */
char pw_run (struct Passwd xdata *p)
{
	switch (p->state) {
	case STATE_GEN_NUMS:
		pw_gen_nums(p);
		break;
	
	case STATE_PROMPT:
		pw_prompt(p);
		break;
		
	case STATE_CHECK:
		pw_check(p);
		break;

	case STATE_AUTH_SUCCESS:
		return 1;

	case STATE_AUTH_FAILED:
	default:
		return -1;
	}

	return 0;
}












