
/*
 *	$Id: queue.c,v 1.2 2002/03/08 17:41:30 klaus Exp $
 *
 *	queue.c		Funktionen zum Verwalten von Queues
 *
 *
 *
 *	(c) 2000,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "queue.h"
#include "buf.h"
#include "system.h"
#include "mm.h"

/*--------------------------------------------------------------------------*/


/*
 *	Initialisiert eine Puffer-Liste
 *
 */
void q_init (struct Queue xdata *q, unsigned char maxcnt)
{
	q->first = q->last = NULL;
	q->roomcnt = q->maxcnt = maxcnt;
}



/*
 *	Loescht alle Puffer aus der Queue *q. Die Queue
 *	selbst wird nicht geloescht, da sie in den meisten
 *	Faellen nicht dynamisch erzeugt wurde.
 *
 */
void q_clear (struct Queue xdata *q)
{
	struct Buffer xdata * idata buf;
	struct Buffer xdata * idata cur;

	cur = q->first;

	while (cur) {
		buf = cur;
		cur = cur->next;

		buf_delete(buf);
	}

	q_init(q, q->maxcnt);
}


/*
 *	Gibt != 0 zurueck, wenn die Queue keine weiteren Puffer
 *	aufnehmen kann. Wenn die Queue Aufnahmebereit ist,
 *	wird 0 zurueckgegeben.
 *
 */
char q_isbusy (struct Queue xdata *q) 
{
	return (q->maxcnt && !q->roomcnt);
}


/*
 *	Haengt den Puffer *buf ans Ende der Queue. Wenn die
 *	Queue voll ist, wird der Puffer nicht eingereiht und
 *	es wird 0 zurueckgegeben. Nach erfolgreichem Einreihen
 *	wird 1 zurueckgegeben.
 *
 */
char q_enqueue (struct Queue xdata *q,struct Buffer xdata *buf)
{
	if (q_isbusy(q))
		return 0;
	
	buf->next = NULL;

	if (q->first == NULL) {
		q->first = buf;
		q->last  = buf;
	} else {
		q->last->next = buf;
		q->last = buf;
	}

	q->roomcnt--;
	return 1;
}



/*
 *	Haengt den ersten Puffer aus der Queue aus und gibt einen
 *	Zeiger auf ihn zurueck. Ist die Queue leer, so wird NULL
 *	zurueckgegeben.
 *
 */
struct Buffer xdata * q_dequeue (struct Queue xdata *q)
{
	struct Buffer xdata * idata buf;

	if (!q->first)
		return NULL;

	buf = q->first;

	if ( (q->first = buf->next) == NULL )
		q->last = NULL;
	
	q->roomcnt++;

	return buf;
}


