
#ifndef _SCC_H_
#define _SCC_H_

#include "system.h"
#include "page.h"

/*
 *	Zeitkonstanten in T2-Ticks
 *
 */
#define SCC_TC_10MS	4608	/* 10ms */
#define SCC_TC_512BPS	900	/* Bitlaenge bei 512bps */
#define SCC_TC_1200BPS	384	/* Bitlaenge bei 1200bps */
#define SCC_TC_2400BPS	192	/* Bitlaenge bei 2400bps */

/*
 *	Zeitkonstanten, mit denen T0 initialisiert werden
 *	muss, damit der Timer genau 3/4 Bitlaenge laeuft.
 *
 */
#define SCC_TC_512_75	0xfaba
#define SCC_TC_1200_75	0xfdc0
#define SCC_TC_2400_75	0xfee0
#define SCC_TC_AS	SCC_TC_2400_75

/*
 *	Umrechnung von einer Geschwindigkeit (in bps) zu
 *	einer Zeitkonstanten
 *
 */
#define SCC_BPS2TC(s)	(57600/(((unsigned int)(s)) >> 3))

/*
 *	Datenraten, die vom SCC-Modul unterstuetzt werden.
 *
 *	ACHTUNG: Diese Definitionen muessen mit der Timecode-
 *		 Tabelle tc_tab[] synchronisiert und der
 *		 decoder-tabelle dectab[] (dectabs.c)
 *		 synchronisiert sein.
 *
 */
#define SCC_B512	0
#define SCC_B1200	1
#define SCC_B2400	2

#define SCC_BAUTO	0x0a

#define SCC_BMIN	SCC_B512
#define SCC_BMAX	SCC_B2400
#define SCC_BCNT	(SCC_BMAX - SCC_BMIN + 1)

/*
 *	Synchron- und Idle-Codewoerter nach der
 *	POCSAG spezifikation. (Das MSB wird zuerst
 *	ausgesendet!)
 *
 */
#define CW_SYNC		0x7cd215d8UL
#define CW_IDLE         0x7a89c197UL

/*
 *	Anzahl Codewoerter und Frames in einem Batch
 *
 */
#define CWS_PER_FRAME		2
#define FRAMES_PER_BATCH	8

/*
 *	Default-Parameter
 *
 */
#define SCC_DEFAULT_TXHEAD	1	/* 10ms */
#define SCC_DEFAULT_TXTAIL	1	/* 10ms */
#define SCC_DEFAULT_TXINVERT	0	/* off */
#define SCC_DEFAULT_RXINVERT	0	/* off */
#define SCC_DEFAULT_PREAMBLE	576	/* Laenge der Praeambel in Bits */


#ifdef SDCC
void scc_ct0_isr () interrupt INT_CT0 using REGB_HP;
void scc_cm0_isr () interrupt INT_CM0 using REGB_HP;
void scc_cm1_isr () interrupt INT_CM1 using REGB_HP;
#endif

void scc_init ();
void scc_recv_mode (unsigned char on);
char scc_recv (unsigned long xdata *buf, unsigned int size);
unsigned int scc_recv_poll (unsigned char xdata *speed);

char scc_xmit (unsigned char speed, const struct PageQueue xdata *pq);
char scc_calibrate (unsigned int bps, unsigned char duration);
char scc_xmitr_state ();

#endif

