
#ifndef _SCHED_H_
#define _SCHED_H_

#include "ax25.h"
#include "scc.h"
#include "page.h"
#include "mm.h"

/*
 *	Definitionen, die die Grenzen der Zeitsteuerung abstecken:
 *
 *	Die Zeit wird in 64 Zyklen zu 16 Slots zu je 6.4 Sekunden aufgeteilt. 
 * 	Die Werte ergeben sich direkt aus der Maskierung der Systemzeit.
 *
 */
#define SCHED_CYCL_CNT		64	/* Anzahl Zyklen */
#define SCHED_SLOT_CNT		16	/* Anzahl Sende-Slots */
#define SCHED_SLOT_DURATION	64	/* Slotdauer in 1/10 Sekunden */

#define SCHED_CYCL_MASK		0xfc00	/* aktuelle Zyklus-Nummer */
#define SCHED_SLOT_MASK		0x03c0  /* aktuelle Slotnummer */
#define SCHED_ELAP_MASK		0x003f	/* verstrichene Slotzeit in 1/10-Sekunden */

#define SCHED_ELAP(t)		((t)  & SCHED_ELAP_MASK)
#define SCHED_SLOT(t)		(((t) & SCHED_SLOT_MASK) >> 6)
#define SCHED_CYCL(t)		(((t) & SCHED_CYCL_MASK) >> 10)

/*
 *	Vom Scheduler unterstuetzte Baudraten. Achtung, mit den
 *	entsprechenden Werten wird auch die Page-Queue-Tabelle
 *	<pqtab[]> indiziert!
 *
 */
#define SCHED_SPEED_512		SCC_B512
#define SCHED_SPEED_1200	SCC_B1200
#define SCHED_SPEED_2400	SCC_B2400

#define SCHED_SPEED_MIN		SCC_BMIN
#define SCHED_SPEED_MAX		SCC_BMAX
#define SCHED_SPEED_CNT		SCC_BCNT

/*
 *	Unterhalb von SCHED_BUSY_MARK Bytes freiem Speicher,
 *	nimmt der Scheduler keine weiteren Funkrufe entgegen.
 *	Das soll verhindern, dass die Funkruf-Queues den
 *	gesamten Speicher belegen und somit das System
 *	ersticken.
 *
 */
#define SCHED_BUSY_MARK		(MEMSIZE / 2)


struct Slot {
	/*
	 *	!= 0 => SCC darf in diesem Slot senden.
	 *
	 */
	unsigned char		tx_unlock;

	/*
	 *	Anzahl der aufeinanderfolgenden Sendeslots (Slots,
	 *	in denen gesendet werden darf)
	 *
	 */
	unsigned char		seq_len;
};


void sched_init ();

void sched_set (unsigned int slot_mask);
void sched_lock (unsigned int slot_mask);
void sched_unlock (unsigned int slot_mask);

char sched_addpage (unsigned char speed, struct Page xdata *page,
		    unsigned char head);
		    
void sched_print_state (struct AX25_cb xdata *axcb);
void sched_print_qstate (struct AX25_cb xdata *axcb);

void sched_run ();

#endif

