
/*
 *	$Id: serial.c,v 1.1.1.1 2002/02/15 22:36:40 Klaus Exp $
 *
 *	serial.c	Funktionen, ueber die auf die serielle
 *			Schnittstelle zugegriffen wird.
 *
 *			Da sowohl von der Interrupt-Routine und
 *			den normalen Funktionen auf die Ringpuffer
 *			zugegriffen wird, muss darauf geachtet
 *			werden, dass die Ringpufferzeigeroperationen
 *			meistens atomar sein muessen! Ein Entfernen
 *			der ES0=0;...;ES0=1 Paerchen fuehrt zwangslaeufig
 *			zu schwer reproduzierbaren Fehlern...
 *			Also bitte nur aendern, wenn man genau weiss,
 *			was man tut!!!
 *
 *
 *
 *	(c) 1999,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "serial.h"
#include "system.h"

/*
 *	Empfangsringpuffer
 *
 */
static unsigned char xdata rx_buf[SER_RXBUFSIZE];
static unsigned char xdata * idata rx_rptr;
static unsigned char xdata * idata rx_wptr;

/*
 *	Senderingpuffer
 *
 */
static unsigned char xdata tx_buf[SER_TXBUFSIZE];
static unsigned char xdata * idata tx_rptr;
static unsigned char xdata * idata tx_wptr;
static volatile unsigned char idata tx_buflen;

/*
 *	Flag zeigt an, ob der Sender laeuft.
 *
 */
#ifdef SDCC
static volatile unsigned char idata xmit;
#else
static volatile bit xmit;
#endif


/*
 *	Makros zum Inkrementieren der Ringpuffer-Zeiger
 *
 *	ACHTUNG!! Es darf nur mit diesen Makros auf die Ringpuffer
 *		  zugegriffen werden!!
 *
 */
#define INCRXPTR(ptr)	{						\
				if ((ptr) == rx_buf + SER_RXBUFSIZE - 1)\
					(ptr) = rx_buf;			\
				else					\
					(ptr)++;			\
			}

#define INCTXPTR(ptr)	{						\
				if ((ptr) == tx_buf + SER_TXBUFSIZE - 1)\
					(ptr) = tx_buf;			\
				else					\
					(ptr)++;			\
			}

/*---------------------------------------------------------------------------*/


/*
 *	Initialisiert die Serielle Schnittstelle mit den Einstellungen
 *	19200 bps, 8n1.
 *
 */
void ser_init ()
{
	ES0 = 0;	

	/* Sende und Empfangsringpuffer initialisieren */
	rx_rptr = rx_wptr = rx_buf;
	tx_rptr = tx_wptr = tx_buf;
	tx_buflen = 0;
	xmit = 0;

	S0CON = 0x50;

	TL1 = TH1 = 0xfd;	/* Baudrate von 19k2 einstellen */
	TMOD &= 0x0f;
	TMOD |= 0x20;
	PCON |= 0x80;		/* SMOD = 1 */
	TR1 = 1;		

	PS0 = 0;
	ES0 = 1;
}



/*
 *	Interrupt-Routine wickelt den Datenaustausch zwischen
 *	RX/TX-Ringpuffer und serieller Schnittstelle ab.
 *
 */
void ser_isr () interrupt INT_S0 using REGB_LP
{
	if (RI) {
		unsigned char xdata * idata wptr = rx_wptr;
		
		RI = 0;
		
		*wptr = S0BUF;
		INCRXPTR(wptr);	

		if (wptr != rx_rptr)
			rx_wptr = wptr;
	}

	if (TI) {
		TI = 0;

		if (tx_rptr != tx_wptr) {
			S0BUF = *tx_rptr;
			INCTXPTR(tx_rptr);
			tx_buflen--;
			xmit = 1;
		} else
			xmit = 0;
	}
}



/*
 *	Gibt das naechste Empfangsbyte nach (int) ge-cast-ed zurueck.
 *	Wenn der Empfangspuffer leer ist, wird -1 zurueckgegeben.
 *
 */
int ser_getch ()
{
	unsigned char xdata * idata wptr;
	int idata ch;
	
	ES0 = 0;
	wptr = rx_wptr;
	ES0 = 1;
	
	if (rx_rptr == wptr)
		return -1;

	ch = (int)((unsigned char)*rx_rptr);

	ES0 = 0;
	INCRXPTR(rx_rptr);	
	ES0 = 1;

	return ch;
}



/*
 *	Schiebt das Byte "ch" in den Senderingpuffer. Es wird die
 *	Anzahl der erfolgreich im Sendepuffer abgelegten Bytes
 *	zurueckgegeben (also 0 oder 1).
 *
 */
unsigned char ser_putch (unsigned char ch)
{
	unsigned char xdata * idata wptr;
	unsigned char xdata * idata rptr;

	ES0 = 0;
	rptr = tx_rptr;
	ES0 = 1;

	wptr = tx_wptr;
	INCTXPTR(wptr);

	if (wptr == rptr)
		return 0;

	*tx_wptr = ch;

	ES0 = 0;
	tx_wptr = wptr;
	tx_buflen++;

	if (!xmit)
		TI = 1;
	
	ES0 = 1;

	return 1;
}



/*
 *	Gibt 1 zurueck, wenn kein Zeichen mehr in den TX-Puffer
 *	passt. Ansonsten wird 0 zurueckgegeben.
 *
 */
char ser_istxbusy ()
{
	if (tx_buflen < (SER_TXBUFSIZE - 1))
		return 0;
	
	return 1;
}



