
/*
 *	$Id: string.c,v 1.6 2002/03/19 19:59:44 klaus Exp $
 *
 *	string.c	String- und Speicherverwaltungsroutinen
 *
 *
 *
 *	(c) 2000,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "string.h"
#include "mm.h"

/*---------------------------------------------------------------------------*/


/*
 *	Fuellt den Speicherbereich <buf>/<count> mit dem
 *	Zeichen <b>
 *
 */
void memset_x (void xdata *buf, unsigned char b, int count)
{
	while (count) {
		*((char *)buf) = b;
		buf = (char *)buf + 1;
		
		count--;
	}
}


/*
 *	Kopiert <size> Bytes von <src> nach <dest>
 *
 */
void memcpy_xx (void xdata * dest, const void xdata * src, unsigned int size)
{
	register unsigned char xdata *d = dest;
	register const unsigned char xdata *s = src;

	while (size) {
		*d = *s;
		s++;
		d++;
		size--;
	}
}


/*
 *	Kopiert <size> Bytes von <src> nach <dest>
 *
 */
void memcpy_xc (void xdata * dest, const void code * src, unsigned int size)
{
	register unsigned char xdata *d = dest;
	register const unsigned char code *s = src;

	while (size) {
		*d = *s;
		s++;
		d++;
		size--;
	}
}


/*
 *	Kopiert <src> nach <dest>
 *
 */
void strcpy_xx (char xdata *dest, const char xdata *src)
{
	while (*src)
		*dest++ = *src++;

	*dest = 0;
}


/*
 *	Gibt die Laenge des Strings <str> zurueck.
 *
 */
unsigned int strlen_x (const char xdata * str)
{
	register unsigned int len = 0;

	while (*str++)
		len++;
		
	return len;
}


/*
 *	Sucht das Fragment <frag> im String <str> und gibt
 *	den Zeiger auf das erste Vorkommen zurueck. Wenn <frag>
 *	nicht in <str> gefunden wurde, wird NULL zurueckgegeben.
 *
 */
char xdata * strstr_xx (const char xdata *str, const char xdata *frag)
{
	while (*str)
		if (*str != *frag) {
			str++;
			continue;
		} else {
			const char xdata *s = str + 1;
			const char xdata *f = frag + 1;

			while (*s != 0 && *f != 0) {
				if (*s != *f)
					break;

				s++;
				f++;
			}

			if (*f == 0)
				return (char xdata *)str;

			str++;
		}

	return NULL;
}


/*
 *	Vergleicht zwei Strings und gibt folgendes zurueck:
 *
 *	BEDINGUNG	RUECKGABE
 *	s1 = s2		0
 *	s1 < s2		<0	
 *	s1 > s2		>0
 *
 */
char strcmp_xc (const char xdata *s1, const char code *s2)
{
	while (*s1 == *s2) {
		if (!*s1)
			return 0;
	
		s1++;
		s2++;
	}

	return *s1 - *s2;
}



/*
 *	Wandelt den numerischen String <str> in einen unsigned long um und
 *	gibt ihn zurueck. Die Umwandlung hoert bei dem ersten nicht mehr
 *	interpretierbaren Zeichen auf. Wenn <**endptr> nicht NULL ist, wird
 *	<*endptr> auf das erste nicht mehr interpretierbare Zeichen gesetzt.
 *	Dieser Algorithmus funktioniert nur fuer base = [2..36].
 *
 */
unsigned long strtoul (const char xdata *str, char xdata **endptr,
		       unsigned char base)
{
	unsigned long num = 0;

	for (; *str != 0; str++) {
		unsigned char ch = *str;
		unsigned char digit;

		if (ch >= '0' && ch <= '9')
			digit = ch - '0';
		else if (ch >= 'A' && ch <= 'Z')
			digit = ch - 'A' + 10;
		else if (ch >= 'a' && ch <= 'z')
			digit = ch - 'a' + 10;
		else
			break;

		if (digit >= base)
			break;

		num *= base;
		num += digit;
	}

	if (endptr)
		*endptr = (char xdata *)str;

	return num; 
}


/*
 *	Wandelt einen Bit-String in einen unsigned int. Es wird
 *	der resultierende Wert zurueckgegeben. Die Funktion setzt
 *	<endptr> auf das erste nicht mehr interpretierbare Zeichen.
 *
 *	Ein Bit-String (eigene Erfindung ;-)) ist eine Folge von
 *	Hex-Ziffern, die korrespondierend zu einem 1-Bit in einem
 *	unsigned int sind.
 *
 *	Beispiel:
 *
 *	bstrtoui("01234567f", NULL)
 *
 *	sollte 0x80ff zurueckgeben.
 *	
 */
unsigned int bstrtoui (const char xdata *str, char xdata **endptr)
{
	static unsigned int code bittab[16] = {
		0x0001, 0x0002, 0x0004, 0x0008,
		0x0010, 0x0020, 0x0040, 0x0080,
		0x0100, 0x0200, 0x0400, 0x0800,
		0x1000, 0x2000, 0x4000, 0x8000
	};
	
	unsigned int ui = 0;

	for (; *str != 0; str++) {
		unsigned char ch = *str;
		unsigned char digit;
	
		if (ch >= '0' && ch <= '9')
			digit = ch - '0';
		else if (ch >= 'A' && ch <= 'F')
			digit = ch - 'A' + 10;
		else if (ch >= 'a' && ch <= 'f')
			digit = ch - 'a' + 10;
		else
			break;

		ui |= bittab[digit];
	}
	
	if (endptr)
		*endptr = (char xdata *)str;

	return ui;
}


/*---------------------------------------------------------------------------*/
/*		IMPLEMENTATION EINES EINFACHEN TOKENIZERS	             */
/*---------------------------------------------------------------------------*/

static char xdata *tok_tail;
static const char code *tok_sep;
static bit tok_squeeze;	


/*
 *	Entfernt alle Separator-Zeichen <tok_sep> am Anfang 
 *	von <tok_tail>.
 *
 */
static void strtok_trim ()
{
	while (*tok_tail) {
		const char code *sep;
	
		for (sep = tok_sep; *sep != 0; sep++)
			if (*tok_tail == *sep) {
				if (!tok_squeeze)
					return;
			
				break;
			}

		if (!*sep)
			break;

		tok_tail++;
	}
}


/*
 *	Initialisiert den Tokenizer mit dem zu zerlegenden String <str>
 *	und den Separator-Zeichen <sep>.
 *
 */
void strtok_init (char xdata *str, const char code *sep, char squeeze)
{
	tok_tail = str;
	tok_sep = sep;
	tok_squeeze = squeeze;
}


/*
 *	Trennt das naechste Token vom String ab und gibt einen Zeiger
 *	darauf zurueck. Wenn kein weiteres Token mehr verfuegbar ist,
 *	wird NULL zurueckgegeben.
 *
 */
char xdata * strtok ()
{
	char xdata *token;
	
	strtok_trim();

	if (!*tok_tail)
		return NULL;

	token = tok_tail;

	/*
	 *	tok_tail hinter das Ende des Tokens schieben
	 *
	 */
	while (*tok_tail) {
		const char code *sep;
	
		for (sep = tok_sep; *sep != 0; sep++)
			if (*tok_tail == *sep)
				break;

		if (*sep) {
			*tok_tail++ = 0;
			break;
		}
		
		tok_tail++;
	}

	return token;
}


/*
 *	Gibt den noch nicht zerlegten Teil des Strings zrurueck. Wenn
 *	der Rest nicht mehr zerlegt werden kann bzw. bereits der komplette
 *	String zerlegt wurde, wird NULL zurueckgegeben.
 *
 */
char xdata * strtok_tail ()
{
	strtok_trim();

	if (!*tok_tail)
		return NULL;

	return tok_tail;
}


