
#ifndef _SYSTEM_H_
#define _SYSTEM_H_

#ifdef SDCC
#include <mcs51reg.h>
#else
#include <philips/reg552.h>
#endif


/*
 *	Die Interrupts des 80C552
 *
 *	Entnommen aus dem "Philips Data Handbook IC20" (1995),
 *	Seite 784, unten.
 *
 */
#define	INT_X0		0
#define INT_T0		1
#define INT_X1		2
#define INT_T1		3
#define INT_S0		4
#define INT_S1		5
#define INT_CT0		6
#define INT_CT1		7
#define INT_CT2		8
#define INT_CT3		9
#define INT_ADC		10
#define INT_CM0		11
#define INT_CM1		12
#define INT_CM2		13
#define INT_T2		14

/*
 *	In dieser Firmware bekommen ISRs der jeweiligen Prioritaet
 *	bestimmte Registerbaenke zugeordnet...
 *
 */
#define REGB_LP		1	/* low priority */
#define REGB_HP		2	/* high priority */


/*
 *	Watchdog-Timer zuruecksetzten
 *
 */
#define WATCHDOG()	{ 			\
				PCON |= 0x10;	\
				T3 = 0;		\
			}
			

/*
 *	Fuehrt einen Reset des Systems aus
 *
 */
#define RESET()		{ 			\
				EA = 0; 	\
				for(;;); 	\
			}



/*
 *	Bitmasken der LED-Bits auf Port 4. Das entsprechende
 *	LED-Bit muss auf 0 gesetzt werden, um die LED einzuschalten!
 *
 */
#define LED_ACT		0x02
#define LED_DCD		0x04
#define LED_CON		0x08
#define LED_PTT		0x10

/*
 *	PTT-Leitung ist mit der PTT-LED gekoppelt...
 *
 */
#define PTT		LED_PTT

/*
 *	Bitmaske der Sendedatenleitung im RTE und STE-Register
 *
 */
#define TXD		0x01

/*
 *	Bitmaske der RXD-Leitung, die an Port 1 angeschlossen ist.
 *
 */
#define RXD		0x01

/*
 *	Makro zur Abfrage des TEST-Tasters
 *
 */
#define TEST_KEY	((~P5) & 0x10)


/*
 *	Makros zum Ein- bzw. Ausschalten der unterschiedlichen LEDs
 *
 *	Wenn diese Makros auf LED_PTT angewendet werden, wird
 *	gleichzeitig die PTT-Leitung beeinflusst!!!!
 *
 */

#define ON(led)		P4 &= ~(led)
#define OFF(led)	P4 |= (led)
#define FLIP(led)	P4 ^= (led)
#define IS_ON(led)	(!(P4 & (led)))

/*
 *	Makros zum einblenden der unterschiedlichen Speicherbausteine
 *	in die entsprechenden Segmente.
 *
 */
#define CSEG_CODE()	P3 |= 0x20
#define CSEG_FLASH()	P3 &= ~0x20
#define XSEG_XDATA()	P3 |= 0x10
#define XSEG_FLASH()	P3 &= ~0x10

#define IS_CSEG_CODE()	(P3 & 0x20)
#define IS_CSEG_FLASH() ((~P3) & 0x20)
#define IS_XSEG_XDATA() (P3 & 0x10)
#define IS_XSEG_FLASH() ((~P3) & 0x10)
 
#define FLASH_A16(b)	{				\
				if ((b))		\
					P4 |= 0x80;	\
				else			\
					P4 &= ~0x80;	\
			}

#define FLASH_A16_STATUS()	(P4 & 0x80)
				

/*
 *	Systemnahe Funktionen
 *
 */
void sys_hwm_init ();
unsigned char sys_hwm ();
unsigned int sys_cli ();
void sys_sti (unsigned int mask);


#endif /* _SYSTEM_H_ */

