
/*
 *	$Id: xos.c,v 1.2 2002/03/08 20:53:23 Klaus Exp $
 *
 *	xos.c		Hauptdatei der Firmware
 *
 *
 *
 *	(c) 2000,	Klaus Hennemann, DL3KHB
 *
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2, or (at your option) any
 *	later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	In other words, you are welcome to use, share and improve this program.
 *	You are forbidden to forbid anyone else to use, share and improve
 *	what you give them.   Help stamp out software-hoarding!  
 */

#include "system.h"
#include "mm.h"
#include "serial.h"
#include "kiss.h"
#include "ax25_sup.h"
#include "rtc.h"
#include "timer.h"
#include "infobox.h"
#include "scc.h"
#include "eeprom.h"
#include "config.h"
#include "listener.h"
#include "sched.h"
#include "scp.h"
#include "boot.h"
#include "flash.h"
#include "master.h"
#include "repeater.h"

#ifdef SDCC
static unsigned char xdata spare;
#endif

/*---------------------------------------------------------------------------*/


/* 
 *	Laesst die LEDs DCD, ACT und CON ein paar mal
 *	blinken.
 *
 */
void user_notify ()
{
	struct Timer xdata t;
	unsigned char n;

	t_set(&t,1,T_RES100MS);

	for (n = 0; n < 10; n++) {
		FLIP(LED_DCD);
		FLIP(LED_ACT);
		FLIP(LED_CON);

		t_start(&t);

		while (!t_isexpired(&t)) {
			WATCHDOG();
			t_run();
		}

		t_stop(&t);
	}

	OFF(LED_DCD);
	OFF(LED_ACT);
	OFF(LED_CON);
}



void main ()
{
	WATCHDOG();

	sys_hwm_init();
	mm_init();
	ee_init();
	flash_init();

	WATCHDOG();

	EA = 1;
	boot();
	EA = 0;
	
	ser_init();
	kiss_init();
	rtc_init();
	scc_init();
	sched_init();

	EA = 1;

	user_notify();
	
	WATCHDOG();
	conf_init();

	WATCHDOG();
	rp_init();

	for(;;) {
		WATCHDOG();
		kiss_run();
		
		WATCHDOG();
		ax25_run();
	
		WATCHDOG();
		t_run();
		
		WATCHDOG();
		ibox_run();

		WATCHDOG();
		l_run();

		WATCHDOG();
		scp_run();

		WATCHDOG();
		sched_run();

		WATCHDOG();
		m_run();

		WATCHDOG();
		rp_run();
	}
}


